library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.std_logic_textio.all;
use std.textio.all;



entity axis_assert is
    
    generic (
        width                : integer := 64;
        axis_dbguser         : boolean := true;        
        axis_dbglast         : boolean := true;
        tcq                  : integer := 5000 );
    port (
        clk                  : in  std_logic;
        reset                : in  std_logic;
        
        s_axis_data          : in  std_logic_vector(width-1 downto 0);
        s_axis_valid         : in  std_logic;
        s_axis_user          : in  std_logic;
        s_axis_last          : in  std_logic;        
        ss_axis_ready        : in  std_logic;
        
        m_axis_ready         : in  std_logic );
    
end axis_assert;	


architecture rtl of axis_assert is

    signal s_axis_data_r         : std_logic_vector(width-1 downto 0);
    signal s_axis_valid_r        : std_logic;    
    signal s_axis_user_r         : std_logic;
    signal s_axis_last_r         : std_logic;
    signal ss_axis_ready_r       : std_logic;
    signal axis_user_next        : std_logic;
    signal axis_user_next_r      : std_logic := '0';
    
    
            function or_reduce (x : std_logic_vector) return std_logic is
    variable result : std_logic := '0';
  begin

    for i in x'range loop
      result := result or x(i);
    end loop;
    return result;
  end or_reduce; 
    
begin
    --synthesis translate_off    

    axis_user_next <= '1' when (s_axis_valid and s_axis_last and ss_axis_ready) = '1' else
                      '0' when (s_axis_valid and s_axis_user and ss_axis_ready) = '1' else axis_user_next_r;
    process (clk) is
    begin
        if rising_edge(clk) then
            if (reset = '1') then
                s_axis_data_r    <= s_axis_data        after (tcq) * 1 ps;
                s_axis_valid_r   <= '0'                after (tcq) * 1 ps;                
                s_axis_user_r    <= '0'                after (tcq) * 1 ps;
                s_axis_last_r    <= '0'                after (tcq) * 1 ps;
                ss_axis_ready_r  <= '0'                after (tcq) * 1 ps;
                axis_user_next_r <= '0'                after (tcq) * 1 ps;                
            else
                s_axis_data_r    <= s_axis_data        after (tcq) * 1 ps;
                s_axis_valid_r   <= s_axis_valid       after (tcq) * 1 ps;                
                s_axis_user_r    <= s_axis_user        after (tcq) * 1 ps;
                s_axis_last_r    <= s_axis_last        after (tcq) * 1 ps;
                ss_axis_ready_r  <= ss_axis_ready      after (tcq) * 1 ps;
                axis_user_next_r <= axis_user_next     after (tcq) * 1 ps;
            end if;                        
        end if;
    end process;

    -- asserts
    process (clk)
    begin
        if falling_edge(clk) then
            if (reset = '0') then                
                if (s_axis_valid and ss_axis_ready and axis_user_next_r) = '1' then
                    if (s_axis_user /= '1' and axis_dbguser = true) then
           --             report "WARNING: Custom axis scheme - s_axis_user expected. Probably illegal." severity failure;
                    end if;
                end if;

                if s_axis_valid /= '0' and s_axis_valid /= '1' then
                    report "WARNING: s_axis_valid illegal(x)." severity failure;
                end if;
                
                if m_axis_ready /= '0' and m_axis_ready /= '1' then
                    report "WARNING: m_axis_ready illegal(x)." severity failure;
                end if;

                if (s_axis_valid_r and (not s_axis_valid) and (not ss_axis_ready_r)) = '1' then
                    report "ERROR: s_axis_valid deasserted illegally." severity failure;
                end if;

                if (s_axis_valid = '1') then
                    if (s_axis_last /= '0' and s_axis_last /= '1'  and axis_dbglast = true) then
                        report "WARNING: s_axis_last illegal(x)." severity failure;
                    end if;

                    if (s_axis_user /= '0' and s_axis_user /= '1' and axis_dbguser = true) then
                        report "WARNING: s_axis_user _probably_ illegal(x)." severity failure;
                    end if;

                    if (or_reduce(s_axis_data) /= '0' and or_reduce(s_axis_data) /= '1') then
                        report "WARNING: s_axis_data _probably_ illegal(x)." severity failure;
                    end if;                                        
                end if;

                if (s_axis_valid_r and s_axis_valid and (not ss_axis_ready_r)) = '1' then
                    if (s_axis_data /= s_axis_data_r) then



                       report "ERROR: s_axis_data changed illegally." severity failure;
                    end if;
                    if (s_axis_last /= s_axis_last_r and axis_dbglast = true) then
                       report "ERROR: s_axis_last changed illegally." severity failure;
                    end if;
                    if (s_axis_user /= s_axis_user_r and axis_dbguser = true) then
                        report "CRITICAL WARNING: Custom axis scheme - s_axis_user changed illegally." severity failure;
                    end if;
                end if;
            end if;
        end if;
    end process;
    

    --synthesis translate_on
end architecture;    

