library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--library synplify;
--use synplify.all;
--
--library smartfusion2;
--use smartfusion2.all;


entity dpram_generic is

  generic (
    g_addr_width : natural := 15;
    g_data_width : natural := 42
  
    );
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;
    clk_ram_i : in std_logic_vector (39 downto 0);
    
    da_i  : in std_logic_vector(g_data_width-1 downto 0);
    db_i  : in std_logic_vector(g_data_width-1 downto 0);
    aa_i  : in std_logic_vector(g_addr_width-1 downto 0);
    ab_i  : in std_logic_vector(g_addr_width-1 downto 0);
    wa_i  : in std_logic_vector (3 downto 1);
    wb_i  : in std_logic;
    ra_i  : in std_logic;
    rb_i  : in std_logic;

    lock_a_req_i   : in  std_logic;
    lock_b_req_i   : in  std_logic;
    lock_a_grant_o : out std_logic;
    lock_b_grant_o : out std_logic;

    qa_o      : out std_logic_vector(g_data_width-1 downto 0);
    qb_o      : out std_logic_vector(g_data_width-1 downto 0);
    valid_a_o : out std_logic;
    valid_b_o : out std_logic


    );
end dpram_generic;


architecture rtl of dpram_generic is
  component RAM1K18
--    generic (MEMORYFILE:string := "");

    port(A_DOUT         : out std_logic_vector(17 downto 0);
          B_DOUT        : out std_logic_vector(17 downto 0);
          BUSY          : out std_logic;
          A_CLK         : in  std_logic                     := 'U';
          A_DOUT_CLK    : in  std_logic                     := 'U';
          A_ARST_N      : in  std_logic                     := 'U';
          A_DOUT_EN     : in  std_logic                     := 'U';
          A_BLK         : in  std_logic_vector(2 downto 0)  := (others => 'U');
          A_DOUT_ARST_N : in  std_logic                     := 'U';
          A_DOUT_SRST_N : in  std_logic                     := 'U';
          A_DIN         : in  std_logic_vector(17 downto 0) := (others => 'U');
          A_ADDR        : in  std_logic_vector(13 downto 0) := (others => 'U');
          A_WEN         : in  std_logic_vector(1 downto 0)  := (others => 'U');
          B_CLK         : in  std_logic                     := 'U';
          B_DOUT_CLK    : in  std_logic                     := 'U';
          B_ARST_N      : in  std_logic                     := 'U';
          B_DOUT_EN     : in  std_logic                     := 'U';
          B_BLK         : in  std_logic_vector(2 downto 0)  := (others => 'U');
          B_DOUT_ARST_N : in  std_logic                     := 'U';
          B_DOUT_SRST_N : in  std_logic                     := 'U';
          B_DIN         : in  std_logic_vector(17 downto 0) := (others => 'U');
          B_ADDR        : in  std_logic_vector(13 downto 0) := (others => 'U');
          B_WEN         : in  std_logic_vector(1 downto 0)  := (others => 'U');
          A_EN          : in  std_logic                     := 'U';
          A_DOUT_LAT    : in  std_logic                     := 'U';
          A_WIDTH       : in  std_logic_vector(2 downto 0)  := (others => 'U');
          A_WMODE       : in  std_logic                     := 'U';
          B_EN          : in  std_logic                     := 'U';
          B_DOUT_LAT    : in  std_logic                     := 'U';
          B_WIDTH       : in  std_logic_vector(2 downto 0)  := (others => 'U');
          B_WMODE       : in  std_logic                     := 'U';
          SII_LOCK      : in  std_logic                     := 'U'
          );
  end component;

  signal qa, qb, da, db : std_logic_vector (g_data_width downto 0);

  
  signal addr_match                     : std_logic;

  signal lock_a_req, lock_b_req : std_logic;
  signal lock_contention        : std_logic;

  signal lock_a_grant, lock_b_grant, lock_a_grant_d, lock_b_grant_d : std_logic;

  signal wea, web : std_logic_vector (39 downto 0);
 
  
  signal qa_discard_32k, qb_discard_32k : std_logic_vector (17 downto 2);
    signal qa_discard_64k, qb_discard_64k : std_logic_vector (17 downto 1);
    
    signal rea, reb : std_logic;
 signal wa_voted, wb_voted : std_logic;   
attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";
begin
  


  addr_match <= '1' when (aa_i (7 downto 2) = ab_i (7 downto 2)) else '0';  --to save LUTs, overhead
                                        --is only 0.5% (random address)
  da         <= "0" & da_i;
  db         <= "0" & db_i;
  qa_o       <= qa (g_data_width-1 downto 0);
  qb_o       <= qb (g_data_width-1 downto 0);


  wa_voted <= ( (wa_i(1) and wa_i(2)) or (wa_i(1) and wa_i(3)) or (wa_i(2) and wa_i(3))) ;
  
  process (lock_b_grant, lock_a_grant, addr_match, ra_i, rb_i, wa_voted, wb_i)
    begin
      
      if (lock_b_grant = '1') then
        valid_b_o <= '1';
    else
      valid_b_o <= '0' when (((rb_i and wa_voted) or lock_a_grant) and addr_match) else '1';
  end if;
  
  if (lock_a_grant = '1') then
      valid_a_o <= '1';
else
  valid_a_o <= '0' when (((ra_i and wb_i) or lock_b_grant) and addr_match) else '1';
end if;
end process;
  
  
tmr_signals : for i in 0 to 39 generate
  wea (i) <= ( (wa_i(1) and wa_i(2)) or (wa_i(1) and wa_i(3)) or (wa_i(2) and wa_i(3))) and valid_a_o;
  -- "web" signal not TMRed (it requires a block TMR of the scrubber)
  web (i) <= wb_i and valid_b_o;
end generate;

  


  lock_a_grant_o <= lock_a_grant;
  lock_b_grant_o <= lock_b_grant;

  lock_contention <= addr_match and lock_a_req_i and lock_b_req_i;



  lock_a_req <= lock_a_req_i;
  lock_b_req <= lock_b_req_i;

assert ( (lock_a_grant_o and lock_b_grant_o and addr_match) = '0') report "error grant" severity error;


  process (clk_i, rst_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        lock_a_grant_d <= '0';
        lock_b_grant_d <= '0';
    else
      lock_a_grant_d <= lock_a_grant;
      lock_b_grant_d <= lock_b_grant;

    end if;
  end if;
  end process;

  -- lock_a/b_req signal, after the write is done, will always be deasserted for at least one clock cycle
  
  process (lock_contention, lock_a_req, lock_b_req, lock_a_grant_d, lock_b_grant_d)
  begin
    
    if (lock_contention = '0') then
      lock_a_grant <= lock_a_req;
      lock_b_grant <= lock_b_req;
    else
      if (lock_a_grant_d = '0' and lock_b_grant_d = '0') then
        if (lock_b_req = '1' and lock_a_req = '0') then
          lock_b_grant <= '1';
          lock_a_grant <= '0';
        elsif (lock_a_req = '1') then
         lock_a_grant <= '1';
         lock_b_grant <= '0';
       else
         lock_a_grant <= '0';
         lock_b_grant <= '0';
       end if;
    else
      lock_a_grant <= lock_a_grant_d;
      lock_b_grant <= lock_b_grant_d;
    end if;
    end if;
    
  end process;


   gen_32k_daec : if (g_addr_width = 15 and g_data_width = 42) generate

    gen_bram : for i in 1 to 5 generate
      gen_byte : for k in 0 to 3 generate
                sram_two_bit : RAM1K18 
            
                port map(
          A_WIDTH       => "001",
          B_WIDTH       => "001",
          A_WMODE       => '1',  --write data appears on the corresponding read data port
          B_WMODE       => '1',  --write data appears on the corresponding read data port
          A_DOUT_LAT    => '1',
          B_DOUT_LAT    => '1',
          A_EN          => '1',
          B_EN          => '1',
          A_ARST_N      => '1',
          B_ARST_N      => '1',
          A_DOUT_EN     => '1',
          B_DOUT_EN     => '1',
          A_DOUT_SRST_N => '1',
          B_DOUT_SRST_N => '1',
          A_DOUT_ARST_N => '1',
          B_DOUT_ARST_N => '1',
          SII_LOCK      => '1',
          A_DOUT_CLK    => '1',
          B_DOUT_CLK    => '1',

          A_CLK => clk_ram_i (8*(i-1)+k),
          B_CLK => clk_ram_i (8*(i-1)+k),

          A_WEN (1) => '0',
          A_WEN (0) => wea (8*(i-1)+2*k),
          B_WEN (1) => '0',
          B_WEN (0) => web (8*(i-1)+2*k),

          A_DIN (1 downto 0)  => da(8*(i-1)+2*k+1 downto 8*(i-1)+2*k) ,
          A_DIN (17 downto 2) => (others => '0'),
          B_DIN (1 downto 0)  => db(8*(i-1)+2*k+1 downto 8*(i-1)+2*k),
          B_DIN (17 downto 2) => (others => '0'),

          A_ADDR (0 downto 0)  => "0",
          A_ADDR (13 downto 1) => aa_i(14 downto 2),
          B_ADDR (0 downto 0)  => "0",
          B_ADDR (13 downto 1) => ab_i(14 downto 2),


          A_BLK => (others => '1'),
          B_BLK  => (others => '1'),
          
          
          A_DOUT (17 downto 2) => qa_discard_32k (17 downto 2),
          A_DOUT (1 downto 0) => qa(8*(i-1)+2*k+1 downto 8*(i-1)+2*k),
          b_DOUT (17 downto 2) => qb_discard_32k (17 downto 2),
          B_DOUT (1 downto 0) => qb(8*(i-1)+2*k+1 downto 8*(i-1)+2*k)

          );

      end generate gen_byte;

    end generate gen_bram;

        sram_two_bit : RAM1K18   
       
        port map(
          A_WIDTH       => "001",
          B_WIDTH       => "001",
          A_WMODE       => '1',  --write data appears on the corresponding read data port
          B_WMODE       => '1',  --write data appears on the corresponding read data port
          A_DOUT_LAT    => '1',
          B_DOUT_LAT    => '1',
          A_EN          => '1',
          B_EN          => '1',
          A_ARST_N      => '1',
          B_ARST_N      => '1',
          A_DOUT_EN     => '1',
          B_DOUT_EN     => '1',
          A_DOUT_SRST_N => '1',
          B_DOUT_SRST_N => '1',
          A_DOUT_ARST_N => '1',
          B_DOUT_ARST_N => '1',
          SII_LOCK      => '1',
          A_DOUT_CLK    => '1',
          B_DOUT_CLK    => '1',

          A_CLK => clk_i,
          B_CLK => clk_i,

          A_WEN (1) => '0',
          A_WEN (0) => wea (39),
          B_WEN (1) => '0',
          B_WEN (0) => web (39),

          A_DIN (1 downto 0)  => da(41 downto 40),
          A_DIN (17 downto 2) => (others => '0'),
          B_DIN (1 downto 0)  => db(41 downto 40),
          B_DIN (17 downto 2) => (others => '0'),

          A_ADDR (0 downto 0)  => "0",
          A_ADDR (13 downto 1) => aa_i(14 downto 2),
          B_ADDR (0 downto 0)  => "0",
          B_ADDR (13 downto 1) => ab_i(14 downto 2),

          A_BLK => (others => '1'),
          B_BLK  => (others => '1'),
         
          A_DOUT (17 downto 2) => qa_discard_32k(17 downto 2),
          A_DOUT (1 downto 0) => qa(41 downto 40),
          b_DOUT (17 downto 2) => qb_discard_32k(17 downto 2),
          B_DOUT (1 downto 0) => qb(41 downto 40)

          );


  end generate gen_32k_daec;

  gen_64k : if g_addr_width = 16 generate

    gen_bram : for i in 1 to 5 generate

      gen_byte : for k in 0 to 7 generate

        sram_bit : RAM1K18
        
         port map(
          A_WIDTH       => "000",
          B_WIDTH       => "000",
          A_WMODE       => '1',  --write data appears on the corresponding read data port
          B_WMODE       => '1',  --write data appears on the corresponding read data port
          A_DOUT_LAT    => '1',
          B_DOUT_LAT    => '1',
          A_EN          => '1',
          B_EN          => '1',
          A_ARST_N      => '1',
          B_ARST_N      => '1',
          A_DOUT_EN     => '1',
          B_DOUT_EN     => '1',
          A_DOUT_SRST_N => '1',
          B_DOUT_SRST_N => '1',
          A_DOUT_ARST_N => '1',
          B_DOUT_ARST_N => '1',
          SII_LOCK      => '1',
          A_DOUT_CLK    => '1',
          B_DOUT_CLK    => '1',

          A_CLK => clk_ram_i (8*(i-1)+k),
          B_CLK => clk_ram_i (8*(i-1)+k),

          A_WEN (1) => '0',
          A_WEN (0) => wea (8*(i-1)+k),
          B_WEN (1) => '0',
          B_WEN (0) => web (8*(i-1)+k),

          A_DIN (0 downto 0)  => da(8*(i-1)+k downto 8*(i-1)+k),
          A_DIN (17 downto 1) => (others => '0'),
          B_DIN (0 downto 0)  => db(8*(i-1)+k downto 8*(i-1)+k),
          B_DIN (17 downto 1) => (others => '0'),

          --A_ADDR (0 downto 0)  => "0",
          A_ADDR (13 downto 0) => aa_i(15 downto 2),
          --B_ADDR (0 downto 0)  => "0",
          B_ADDR (13 downto 0) => ab_i(15 downto 2),


          A_BLK => (others => '1'),
          B_BLK  => (others => '1'),
       
        A_DOUT (17 downto 1) => qa_discard_64k,
          A_DOUT (0 downto 0) => qa(8*(i-1)+k downto 8*(i-1)+k),
                  b_DOUT (17 downto 1) => qb_discard_64k,
          B_DOUT (0 downto 0) => qb(8*(i-1)+k downto 8*(i-1)+k)

          );
      end generate gen_byte;

    end generate gen_bram;
    end generate gen_64k;
    


  end architecture;