library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sdpram_cdc_r is
    
    generic (
        width     : integer := 72;
        log2depth : integer := 10;
        tcq       : integer := 1 );
    port (
        wrclk     : in  std_logic;
        rdclk     : in  std_logic;        

        wren      : in  std_logic;
        wraddr    : in  std_logic_vector(log2depth-1 downto 0);
        rdaddr    : in  std_logic_vector(log2depth-1 downto 0);

        wrdata    : in  std_logic_vector(width-1 downto 0);
        rddata    : out std_logic_vector(width-1 downto 0) );
    
end sdpram_cdc_r;


architecture rtl of sdpram_cdc_r is
    
    type regfile_t is array (integer range <>) of std_logic_vector (width-1 downto 0);
    signal regs        : regfile_t (0 to ((2**log2depth)-1));

    signal rddata_r    : std_logic_vector(width-1 downto 0);
    signal rddata_h    : std_logic_vector(width-1 downto 0);    
    
begin
    
    process (wrclk)
    begin
        if (rising_edge(wrclk)) then            
            if (wren = '1') then
                regs(to_integer(unsigned(wraddr))) <= wrdata                               after (tcq) * 1 ps;
            end if;                        
        end if;
    end process;

    process (rdclk)
    begin
        if (rising_edge(rdclk)) then                
            rddata_r                               <= regs(to_integer(unsigned(rdaddr)))   after (tcq) * 1 ps;
            rddata_h                               <= rddata_r                             after (tcq) * 1 ps;
        end if;
    end process;
    
    -- outputs
    rddata <= rddata_h;
    
end architecture;

    
