`define BAUD_RATE 115200
`define SCALER (50000000 / `BAUD_RATE)

module uart
(
	input reset_i,						/* synthesis  syn_keep = 1 */
	input clk_i,						/* synthesis  syn_keep = 1 */
	output txd_o,						/* synthesis  syn_keep = 1 */
	input [7:0] tx_data_i,         /* synthesis  syn_keep = 1 */
	output reg tx_wdone_o,           /* synthesis  syn_preserve = 1 */
	output reg tx_ready_o, 
	input tx_req_i               /* synthesis  syn_keep = 1 */
)/* synthesis syn_radhardlevel="tmr" */;

parameter S0 = 0;
parameter S1 = 1;
parameter S2 = 2;
parameter S3 = 3;




/**********************************/
/* TX */
/**********************************/

reg clk_tx;			/* synthesis  syn_preserve = 1 */
reg clk_tx_en;		/* synthesis  syn_preserve = 1 */
reg [15:0] tx_div;	/* synthesis  syn_preserve = 1 */
reg [7:0] tx_data_acq; 	/* synthesis  syn_preserve = 1 */
always@(posedge clk_i)
begin
	clk_tx <= 'b0;
	if (clk_tx_en) begin
		tx_div <= tx_div + 1;
		if (tx_div == `SCALER) begin
			clk_tx <= 1'b1;
			tx_div <= 'b0;
		end
	end
	else tx_div <= 'b0;
end

integer tx_state /* synthesis syn_encoding="onehot" */;
reg [10:0] tx_data;			/* synthesis  syn_preserve = 1 */
reg [4:0] tx_data_cnt;		/* synthesis  syn_preserve = 1 */

always@(posedge reset_i or posedge clk_i)
begin
	if (reset_i) begin
		tx_state <= S0;
		tx_data <= {11{1'b1}};
		tx_wdone_o <= 1'b0;
		clk_tx_en <= 1'b0;
		tx_data_cnt <= 'b0;
		tx_ready_o <= 1'b0;
	end
	else case (tx_state)
		S0: begin
			tx_wdone_o <= 1'b0;
			tx_ready_o <= 1'b1;
			if (tx_req_i & tx_ready_o) begin
				tx_state <= S1;
				tx_wdone_o <= 1'b1;
                tx_data_acq <= tx_data_i;
                tx_ready_o <= 1'b0;


			end

		end
		
		S1: begin
			tx_state <= S2;
			tx_ready_o <= 1'b0;
			tx_wdone_o <= 1'b0;
		end
		
		S2: begin
			tx_data <= {1'b1, tx_data_acq, 2'b01};
			clk_tx_en <= 1'b1;
			tx_state <= S3;
		end
		
		S3: begin
			if (clk_tx)	begin
				tx_data <= {1'b0, tx_data[10:1]};
				tx_data_cnt <= tx_data_cnt + 1;
				if (tx_data_cnt == 9) begin
					tx_data_cnt <= 'b0;
					tx_state <= S0;
					clk_tx_en <= 1'b0;
				end
			end
            end
      
            
		default: tx_state <= S0;
	endcase
end
	
assign txd_o = tx_data[0];

endmodule
