library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--
--library smartfusion2;
--use smartfusion2.all;


entity ueth is

  
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;

    --TX
    phy0_tx_clk_i : in  std_logic;
    phy0_txd_o    : out std_logic_vector (3 downto 0);
    phy0_txen_o   : out std_logic;
    --RX
    phy0_rx_clk_i : in std_logic;
    phy0_rxd_i    : in std_logic_vector (3 downto 0);
    phy0_rxdv_i   : in std_logic;
    phy0_rxerr_i  : in std_logic;

    --TX
    phy1_tx_clk_i : in  std_logic;
    phy1_txd_o    : out std_logic_vector (3 downto 0);
    phy1_txen_o   : out std_logic;
    --RX
    phy1_rx_clk_i : in std_logic;
    phy1_rxd_i    : in std_logic_vector (3 downto 0);
    phy1_rxdv_i   : in std_logic;
    phy1_rxerr_i  : in std_logic;
  

    --MIIM
    phy_mdc_o : out std_logic;
    phy_md_i  : in  std_logic;
    phy_md_o  : out std_logic;
    phy_oe_o  : out std_logic;

    irq_o : out std_logic;
    phy_rst_o : out std_logic;
    -- to the Wishbone xbar
    slave_i : in  t_wishbone_slave_in;
    slave_o : out t_wishbone_slave_out;
    master_o : out t_wishbone_master_out;
    master_i : in  t_wishbone_master_in

    );
end entity ueth;


architecture rtl of ueth is

signal rst_phy, phy_rxen, phy_txen : std_logic;
signal mac_rxd, mac_txd, hub_phy0_txd, hub_phy0_rxd, hub_phy1_txd, hub_phy1_rxd : std_logic_vector (3 downto 0);
signal mac_rx_valid, mac_tx_valid, mac_tx_ready, mac_rx_eof, mac_tx_eof : std_logic;

signal hub_phy0_txdv, hub_phy0_tx_ready, hub_phy0_rxdv, hub_phy0_rx_ready : std_logic;
signal hub_phy1_txdv, hub_phy1_tx_ready, hub_phy1_rxdv, hub_phy1_rx_ready : std_logic;

signal hub_phy0_rx_eof, hub_phy1_rx_eof : std_logic;

attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";



  
begin

phy_rst_o <= rst_phy;

inst_mac : entity work.ueth_mac

  
  port map(
  
  
    clk_i => clk_i,
    rst_i => rst_i,

    rst_phy_o => rst_phy,
    phy_rxen_o => phy_rxen,
    phy_txen_o => phy_txen,

    mac_rxd_i => mac_rxd,
    mac_txd_o => mac_txd,
    mac_rx_valid_i => mac_rx_valid,
    mac_tx_valid_o => mac_tx_valid,
    mac_tx_ready_i => mac_tx_ready,
    mac_rx_eof_i => mac_rx_eof,
    mac_tx_eof_o => mac_tx_eof,
    
        irq_o => irq_o,

    slave_i  => slave_i,
    slave_o => slave_o,

    master_o => master_o,
    master_i => master_i,
   
    -- MIIM
    phy_mdc_o => phy_mdc_o,
    phy_md_i => phy_md_i,
    phy_md_o => phy_md_o,
    phy_oe_o => phy_oe_o
    );


  

inst_hub : entity work.ueth_hub 

  
  port map (
  
    
    phy0_txd_o    => hub_phy0_txd,
    phy0_tx_valid_o =>  hub_phy0_txdv,
    phy0_tx_ready_i => hub_phy0_tx_ready,
    phy0_rxd_i    =>  hub_phy0_rxd,
    phy0_rx_valid_i =>  hub_phy0_rxdv,
    phy0_rx_ready_o =>  hub_phy0_rx_ready, -- backpressure
    phy0_rx_eof_i => hub_phy0_rx_eof,
    
    phy1_txd_o    =>  hub_phy1_txd,
    phy1_tx_valid_o =>  hub_phy1_txdv,
    phy1_tx_ready_i => hub_phy1_tx_ready,
    phy1_rxd_i    =>  hub_phy1_rxd,
    phy1_rx_valid_i =>  hub_phy1_rxdv,
    phy1_rx_ready_o =>  hub_phy1_rx_ready, -- backpressure
    phy1_rx_eof_i => hub_phy1_rx_eof,   

    mac_txd_i => mac_txd,
    mac_rxd_o => mac_rxd,
    mac_tx_valid_i => mac_tx_valid,
    mac_rx_valid_o => mac_rx_valid,
    mac_tx_ready_o => mac_tx_ready, -- backpressure
    mac_rx_eof_o => mac_rx_eof,
    mac_tx_eof_i => mac_tx_eof,
    
    clk_i => clk_i,
    rst_i => rst_i

    );

inst_eth_phy0 :  entity work.ueth_uphy
  port map  (

    clk_i   => clk_i,
    rst_i   => rst_i or rst_phy,
    rx_en_i => phy_rxen,
  

    rxd_o =>  hub_phy0_rxd,
    rxdv_o =>  hub_phy0_rxdv,
    rx_ready_i =>  hub_phy0_rx_ready,
    rx_eof_o => hub_phy0_rx_eof,
    
    txd_i =>  hub_phy0_txd,
    txdv_i =>  hub_phy0_txdv,
    tx_ready_o =>  hub_phy0_tx_ready,
    

    --PHY TX
    phy_tx_clk_i => phy0_tx_clk_i,
    phy_txd_o    => phy0_txd_o,
    phy_txen_o   => phy0_txen_o,

    --PHY RX
    phy_rx_clk_i => phy0_rx_clk_i,
    phy_rxd_i    => phy0_rxd_i,
    phy_rxdv_i   => phy0_rxdv_i,
    phy_rxerr_i  => phy0_rxerr_i 
 );






inst_eth_phy1 :  entity work.ueth_uphy
  port map  (

    clk_i   => clk_i,
    rst_i   => rst_i or rst_phy,
    rx_en_i => phy_rxen,
 

    rxd_o =>  hub_phy1_rxd,
    rxdv_o =>  hub_phy1_rxdv,
    rx_ready_i =>  hub_phy1_rx_ready,
    rx_eof_o => hub_phy1_rx_eof,
    
    txd_i =>  hub_phy1_txd,
    txdv_i =>  hub_phy1_txdv,
    tx_ready_o =>  hub_phy1_tx_ready,
    

    --PHY TX
    phy_tx_clk_i => phy1_tx_clk_i,
    phy_txd_o    => phy1_txd_o,
    phy_txen_o   => phy1_txen_o,

    --PHY RX
    phy_rx_clk_i => phy1_rx_clk_i,
    phy_rxd_i    => phy1_rxd_i,
    phy_rxdv_i   => phy1_rxdv_i,
    phy_rxerr_i  => phy1_rxerr_i 
 );



 




    end architecture;
    
