library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--
--library smartfusion2;
--use smartfusion2.all;




entity ueth_hub is

  
  port (
  
    
    phy0_txd_o    : out std_logic_vector (3 downto 0);
    phy0_tx_valid_o : out std_logic;
    phy0_tx_ready_i : in std_logic;
    phy0_rxd_i    : in std_logic_vector (3 downto 0);
    phy0_rx_valid_i: in std_logic;
    phy0_rx_ready_o : out std_logic;
    phy0_rx_eof_i : in std_logic;
   
   
   

    phy1_txd_o    : out std_logic_vector (3 downto 0);
    phy1_tx_valid_o : out std_logic;
    phy1_tx_ready_i : in std_logic;
    phy1_rxd_i    : in std_logic_vector (3 downto 0);
    phy1_rx_valid_i: in std_logic;
    phy1_rx_ready_o : out std_logic;
    phy1_rx_eof_i : in std_logic;
   
   

    mac_txd_i : in std_logic_vector (3 downto 0);
    mac_rxd_o : out std_logic_vector (3 downto 0);
    mac_tx_valid_i : in std_logic;
    mac_rx_valid_o : out std_logic;
    mac_tx_ready_o : out std_logic;
    mac_tx_eof_i : in std_logic;
    mac_rx_eof_o : out std_logic;
    
    clk_i : in std_logic;
    rst_i : in std_logic

    );


 end entity ueth_hub;

 architecture rtl of ueth_hub is


  type mux_states is (connect_none, connect_mac, connect_phy0, connect_phy1);
  signal mux_phy0, mux_phy1, mux_mac : mux_states;

   type fsm_tx_states is (idle, wait_channel, wait_send, wait_ifg);
   signal fsm_mac_tx : fsm_tx_states := idle;
   type fsm_rx_states is (idle, wait_phy0_rx, wait_phy1_rx);
  signal fsm_mac_rx : fsm_rx_states := idle;
  
  signal phy0_rxd_d, phy1_rxd_d : std_logic_vector (3 downto 0);
  signal phy0_rx_valid_d, phy1_rx_valid_d, phy0_rx_valid_d2, phy1_rx_valid_d2 : std_logic;
signal phy0_rx_eof_d, phy1_rx_eof_d : std_logic;
 signal ifg_count : unsigned (7 downto 0);
   signal ifg_good : std_logic;
   
   
   attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";

 begin

    --backpressure to be used with preamble insertion option
    phy0_rx_ready_o <= '1';
    phy1_rx_ready_o <= '1';
     
     --mux phy0 

    process (mux_phy0, phy1_rxd_i, mac_txd_i, phy1_rx_valid_i, mac_tx_valid_i, mac_tx_ready_o, phy1_rx_eof_i) 
    begin
    if mux_phy0 = connect_phy1 then
       phy0_txd_o <= phy1_rxd_i;
       phy0_tx_valid_o <= phy1_rx_valid_i and not phy1_rx_eof_i;
    elsif mux_phy0 = connect_mac then
       phy0_txd_o <= mac_txd_i;
       phy0_tx_valid_o <= mac_tx_valid_i and mac_tx_ready_o; -- and phy0_tx_ready_i;
    else
       phy0_txd_o <= (others => '0');
       phy0_tx_valid_o <= '0';
    end if;
    end process;

         -- mux phy1 

    process (mux_phy1, phy0_rxd_i, phy0_rx_valid_i, mac_txd_i, mac_tx_valid_i, mac_tx_ready_o, phy0_rx_eof_i)
    begin
    if mux_phy1 = connect_phy0 then
       phy1_txd_o <= phy0_rxd_i;
       phy1_tx_valid_o <= phy0_rx_valid_i and not phy0_rx_eof_i;
    elsif mux_phy1 = connect_mac then
       phy1_txd_o <= mac_txd_i;
       phy1_tx_valid_o <= mac_tx_valid_i and mac_tx_ready_o;-- and phy1_tx_ready_i;
    else
       phy1_txd_o <= (others => '0');
       phy1_tx_valid_o <= '0';
    end if;
    end process;


     -- mux mac 

    process (mux_mac, phy0_rxd_d, phy1_rxd_d, phy0_rx_valid_d, phy1_rx_valid_d, phy0_rx_eof_i, phy1_rx_eof_i)
    begin
    if mux_mac= connect_phy0 then
       mac_rxd_o <= phy0_rxd_d;
       mac_rx_valid_o <= phy0_rx_valid_d;
       mac_rx_eof_o <= phy0_rx_eof_d;
    elsif mux_mac  = connect_phy1 then
       mac_rxd_o <= phy1_rxd_d;
       mac_rx_valid_o <= phy1_rx_valid_d;
       mac_rx_eof_o <= phy1_rx_eof_d;
    else
       mac_rxd_o <= (others => '0');
       mac_rx_valid_o <= '0';
       mac_rx_eof_o <= '0';
    end if;
    end process;

process (clk_i)
begin
  if rising_edge(clk_i) then
    phy0_rx_valid_d <= phy0_rx_valid_i;
    phy0_rx_valid_d2 <= phy0_rx_valid_d;
    phy0_rxd_d <= phy0_rxd_i;
    phy1_rx_valid_d <= phy1_rx_valid_i;
    phy1_rx_valid_d2 <= phy1_rx_valid_d;
    phy1_rxd_d <= phy1_rxd_i;   
    phy0_rx_eof_d <= phy0_rx_eof_i;
    phy1_rx_eof_d <= phy1_rx_eof_i;
end if;
end process;

 process (clk_i)
 begin
    if rising_edge(clk_i) then
      if rst_i = '1' then
        ifg_count <= (others => '0');
        ifg_good <= '0';
    else
       if (phy0_tx_valid_o = '1' or phy1_tx_valid_o = '1') then
          ifg_count <= (others => '0');
          ifg_good <= '0';
       elsif ifg_count < x"C0" then
          ifg_count <= ifg_count + 1;
          ifg_good <=  '0';
       else
          ifg_good <= '1';
       end if;
     end if;
   end if;
end process;
    
    
    mac_tx_ready_o <= phy0_tx_ready_i and phy1_tx_ready_i when (fsm_mac_tx = wait_send) else '0';
    
    process (clk_i)
    begin
    if rising_edge(clk_i) then
       case (fsm_mac_tx) is
          when idle=>
             -- safely connect the PHYs TX/RX (i.e. do not connect in the middle of a frame)
             if (phy0_rx_valid_i = '0' and phy0_rx_valid_d = '0') then
                mux_phy1 <= connect_phy0;
              end if;
              if (phy1_rx_valid_i = '0' and phy1_rx_valid_d = '0') then
                 mux_phy0 <= connect_phy1;
              end if;
              
             if (mac_tx_valid_i = '1') then
                fsm_mac_tx <= wait_channel;
              end if;
          when wait_channel =>
                if (phy0_tx_valid_o = '0' and phy1_tx_valid_o = '0' and ifg_good = '1') then
                   fsm_mac_tx <= wait_send;
                   mux_phy0 <= connect_mac;
                   mux_phy1 <= connect_mac;
                   
                end if;
          when wait_send =>
             
             if (mac_tx_eof_i = '1' and mac_tx_valid_i = '1' and mac_tx_ready_o = '1') then
              
              mux_phy0 <= connect_none;
              mux_phy1 <= connect_none;
              fsm_mac_tx <= wait_ifg;
             end if;
           when wait_ifg =>
              mux_phy0 <= connect_none;
              mux_phy1 <= connect_none;
             if (ifg_good = '1') then
               fsm_mac_tx <= idle;
             end if;
         end case;
     end if;
    end process;

        
   process (clk_i)
    begin
    if rising_edge(clk_i) then
      if (rst_i = '1') then
        fsm_mac_rx <= idle;
    else
       case (fsm_mac_rx) is
          when idle=>
             mux_mac <= connect_none;
             
             if (phy0_rx_valid_i = '1') then
                fsm_mac_rx <= wait_phy0_rx;
             elsif (phy1_rx_valid_i = '1') then
                fsm_mac_rx <= wait_phy1_rx;
             end if;
             
          when wait_phy0_rx =>
             mux_mac <= connect_phy0;
             if ( (phy0_rx_valid_i = '0' and phy0_rx_valid_d = '0' and phy0_rx_valid_d2 = '0') and
                  (phy1_rx_valid_i = '0' and phy1_rx_valid_d = '0' and phy1_rx_valid_d2= '0')) then
               fsm_mac_rx <= idle;
             end if;
          when wait_phy1_rx =>
             mux_mac <= connect_phy1;
             if ( (phy0_rx_valid_i = '0' and phy0_rx_valid_d = '0' and phy0_rx_valid_d2 = '0') and
                  (phy1_rx_valid_i = '0' and phy1_rx_valid_d = '0' and phy1_rx_valid_d2 = '0')) then

               fsm_mac_rx <= idle;
             end if;
             
           when others =>
             fsm_mac_rx <= idle;
            
         end case;
     end if;
   end if;
    end process;
       
        
    
 end architecture;

 
