library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--
--library smartfusion2;
--use smartfusion2.all;


entity ueth_miim is

  
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;

    read_i : in std_logic;
    addr_i : in std_logic_vector (4 downto 0);
    done_o : out std_logic;
    reg_o  : out std_logic_vector (15 downto 0);
    


    --MIIM
    phy_mdc_o : out std_logic;
    phy_md_i  : in  std_logic;
    phy_md_o  : out std_logic;
    phy_oe_o  : out std_logic



    );
end entity ueth_miim;


architecture rtl of ueth_miim is

  constant c_PHY_ADDR : std_logic_vector (4 downto 0) := "00000";
  signal reg_phy_addr : std_logic_vector (4 downto 0) := c_PHY_ADDR; 
  signal clk_en       : std_logic;
  signal clk_divider  : unsigned (3 downto 0);
  signal counter : unsigned (4 downto 0);
  signal clk_miim     : std_logic;
  type t_fsm is (idle, preamble, start_0, start_1, opcode_read_0, opcode_read_1, phy_addr, addr, tristate, skip, read_reg); 
  signal reg_addr : std_logic_vector (4 downto 0);
  signal fsm : t_fsm := idle;
  signal reg_phy : std_logic_vector (15 downto 0);
  
  attribute syn_encoding : string; 
attribute syn_encoding of fsm : signal is "safe,onehot";

  attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";
begin

phy_mdc_o <= clk_miim;

  process (clk_i)
  begin
    if rising_edge(clk_i) then
      if rst_i = '1' then
        clk_divider <=  (others => '0');
        clk_miim    <= '1';
		clk_en <= '0';
      else
        clk_en <= '0';
        clk_divider <= clk_divider+1;
        if (clk_divider = 0) then
          --clk_divider <= (others => '0');
          clk_en     <= '1';
          clk_miim   <= not clk_miim;
        end if;
      end if;
    end if;
    end process;


      process (clk_i)
      begin
        if rising_edge (clk_i) then
          if rst_i = '1' then
            fsm    <= idle;
            done_o <= '0';
            phy_oe_o <= '0';
            reg_phy <= (others => '0');
          else
            case (fsm) is
              when idle =>
                 phy_md_o <= '1';
                
                phy_oe_o <= '0';
                if (read_i = '1') then
                  fsm      <= preamble;
                  counter <= (others => '0');
                  reg_addr <= addr_i;
                  done_o <= '0';
                end if;
            when preamble =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o <= '1';
                  phy_oe_o <= '1';   
                  counter <= counter+1;
                  if (counter = 31) then
                  fsm     <= start_0;
                  counter <= (others => '0');
                end if;
                end if;      
              when start_0 =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o <= '0';
                  phy_oe_o <= '1';
                  fsm      <= start_1;
                end if;
                
              when start_1 =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o <= '1';
                  fsm      <= opcode_read_0;
                end if;
                
              when opcode_read_0 =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o <= '1';
                  fsm      <= opcode_read_1;
                end if;
                
              when opcode_read_1 =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o <= '0';
                  fsm      <= phy_addr;
                  reg_phy_addr <= c_PHY_ADDR;
                  counter  <= (others => '0');
                end if;
                
              when phy_addr =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o     <= reg_phy_addr (4);
                  reg_phy_addr <= reg_phy_addr (3 downto 0) & "0";
                  counter      <= counter + 1;
                end if;
                if (counter = 5) then
                  fsm     <= addr;
                  counter <= (others => '0');
                end if;

              when addr =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_md_o     <= reg_addr (4);
                  reg_addr <= reg_addr (3 downto 0) & "0";
                  counter      <= counter + 1;
                end if;
                if (counter = 5) then
                  fsm     <= tristate;
                  counter <= (others => '0');
                end if;
                
              when tristate =>
                if (clk_en = '1' and clk_miim = '0') then
                  phy_oe_o <= '0';
                  fsm      <= skip;
                end if;

              when skip =>
                if (clk_en = '1' and clk_miim = '0') then
                  fsm <= read_reg;
                end if;
                
              when read_reg =>
                if (clk_en = '1' and clk_miim = '0') then
                  reg_phy <= reg_phy (14 downto 0) & phy_md_i;
                  counter <= counter + 1;
                  if (counter = 15) then
                  fsm     <= idle;
                  counter <= (others => '0');
                  done_o  <= '1';
                end if;
                end if;
                
              when others =>
                fsm <= idle;
                
            end case;
          end if;
        end if;
      end process;

      reg_o <= reg_phy;
    end architecture;
