
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--
--library smartfusion2;
--use smartfusion2.all;


entity ueth_uphy is

  
  port (

    clk_i   : in std_logic;
    rst_i   : in std_logic;
    
    rx_en_i : in std_logic;
   

    rxd_o : out std_logic_vector (3 downto 0);
    rxdv_o : out std_logic;
    rx_ready_i : in std_logic;
    rx_eof_o : out std_logic;

    txd_i : in std_logic_vector (3 downto 0);
    txdv_i : in std_logic;
    tx_ready_o : out std_logic;
    

    --PHY TX
    phy_tx_clk_i : in  std_logic;
    
    phy_txd_o    : out std_logic_vector (3 downto 0);
    phy_txen_o   : out std_logic;
   


    --PHY RX
    phy_rx_clk_i : in std_logic;
    
    phy_rxd_i    : in std_logic_vector (3 downto 0);
    phy_rxdv_i   : in std_logic;
    phy_rxerr_i  : in std_logic

    );
end entity ueth_uphy;

architecture rtl of ueth_uphy is
signal rst_tx, rst_tx_d, rst_rx, rst_rx_d : std_logic;
signal fifo_rxd, fifo_txd : std_logic_vector (3 downto 0);
signal fifo_rxdv, fifo_txdv, fifo_txdv_d : std_logic;
signal fifo_tx_ready : std_logic;
signal rx_en, rx_en_d, rx_en_f, rx_abort, rx_on : std_logic;
signal rx_activity : std_logic;
signal phy_rxdv_d, fifo_eof : std_logic;

attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";

begin

  -- PHY's clocks are not running during resets, therefore we use async reset logic
  process (phy_rx_clk_i, rst_i)
  begin
    if rst_i = '1' then
      rst_rx   <= '1';
      rst_rx_d <= '1';
      rx_en_f <= '0';
      rx_en <= '0';
    elsif rising_edge(phy_rx_clk_i) then
      rst_rx_d <= '0';
      rst_rx   <= rst_rx_d;
      rx_en_f <= rx_en_i;
      rx_en <= rx_en_f;
    end if;
  end process;


  process (phy_tx_clk_i, rst_i)
  begin
    if rst_i = '1' then
      rst_tx   <= '1';
      rst_tx_d <= '1';
    elsif rising_edge(phy_tx_clk_i) then
      rst_tx_d <= '0';
      rst_tx   <= rst_tx_d;
    end if;
  end process;
  
  process (phy_tx_clk_i, rst_rx)
  begin
    if rst_tx = '1' then
      fifo_txdv_d <= '0';
    elsif rising_edge(phy_tx_clk_i) then
     fifo_txdv_d <= fifo_txdv;

  end if;
end process;




 
  phy_txd_o       <= fifo_txd;
  phy_txen_o      <= fifo_txdv and fifo_txdv_d;
  fifo_tx_ready <= fifo_txdv_d;       -- one nibble margin to compensate
                                        -- frequency difference






  fifo_rxd  <= phy_rxd_i;
  fifo_rxdv <= (phy_rxdv_i or (phy_rxdv_d and not phy_rxdv_i)) and rx_on;
  fifo_eof <= phy_rxdv_d and not phy_rxdv_i;



  rx_on <= rx_en and not rx_abort;
 


  -- rx_en is active only after rst_i is de-asserted (MAC force rx_en_i to 0 after reset)
  process (phy_rx_clk_i)
  begin
    if rising_edge (phy_rx_clk_i) then
      phy_rxdv_d <= phy_rxdv_i;
      if (rst_rx = '1') then
        rx_en_d <= '0';
        rx_abort <= '0';
      else
        rx_en_d <= rx_en;
        if (rx_en = '1' and rx_en_d = '0' and phy_rxdv_i = '1') then
          rx_abort <= '1';
        end if;
        if (phy_rxdv_i = '0') then
          rx_abort <= '0';
        end if;
    end if;
    end if;
  end process;

    fifo_rx : entity work.axis_fifo_cdc
    
    generic map (
        dwidth         => 4,
        uwidth         => 0,
        log2depth      => 4,
        tcq            => 1 )
    port map (
        s_clk          => phy_rx_clk_i,
        s_reset        => rst_rx or rx_abort,

        m_clk          => clk_i,
        m_reset        => rst_i or not rx_en_i,
        
        s_axis_data    => fifo_rxd,
        s_axis_valid   => fifo_rxdv,
        s_axis_user    => (others => '0'),
        s_axis_last    => fifo_eof,
        s_axis_ready   => open,
      
        m_axis_data    => rxd_o,
        m_axis_valid   => rxdv_o,
        m_axis_user    => open,
        m_axis_last    => rx_eof_o,
        m_axis_ready   => rx_ready_i);


    fifo_tx : entity work.axis_fifo_cdc
    
    generic map (
        dwidth         => 4,
        uwidth         => 0,
        log2depth      => 4,
        tcq            => 1 )
    port map (
        s_clk          => clk_i,
        s_reset        => rst_i, -- or not tx_en_i,

        m_clk          => phy_tx_clk_i,
        m_reset        => rst_tx,
        
        s_axis_data    => txd_i,
        s_axis_valid   => txdv_i,
        s_axis_user    => (others => '0'),
        s_axis_last    => '0',
        s_axis_ready   => tx_ready_o,
       
        m_axis_data    => fifo_txd,
        m_axis_valid   => fifo_txdv,
        m_axis_user    => open,
        m_axis_last    => open,
        m_axis_ready   => fifo_tx_ready);
      
  end architecture;
  
