
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--
--library smartfusion2;
--use smartfusion2.all;


entity urv_core_DTMR is

      generic (
      g_m2s090 : boolean := true;
      g_dram_data_width : natural := 39;
      g_dram_addr_width : natural := 16
      );
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;

    -- to the IRAM primitive
    d_iram_i          : in  std_logic_vector (38 downto 0);
    q_iram_o          : out std_logic_vector (38 downto 0);
    a_iram_o          : out std_logic_vector (15 downto 0);
    we_iram_o         : out std_logic;
    re_iram_o         : out std_logic;
    valid_iram_i      : in  std_logic;
    lock_iram_req_o   : out std_logic;
    lock_iram_grant_i : in  std_logic;


--    d_iram_bank1_i          : in  std_logic_vector (41 downto 0);
--    q_iram_bank1_o          : out std_logic_vector (41 downto 0);
--    we_iram_bank1_o         : out std_logic;
--    re_iram_bank1_o         : out std_logic;
--    valid_iram_bank1_i      : in  std_logic;
--    lock_iram_req_bank1_o   : out std_logic;
--    lock_iram_grant_bank1_i : in  std_logic;


    -- to the DRAM primitive
    d_dram_i          : in  std_logic_vector (g_dram_data_width-1 downto 0);
    q_dram_o          : out std_logic_vector (g_dram_data_width-1 downto 0);
    a_dram_o          : out std_logic_vector (g_dram_addr_width-1 downto 0);
    we_dram_o         : out std_logic;
    re_dram_o         : out std_logic;
    valid_dram_i      : in  std_logic;
    lock_dram_req_o   : out std_logic;
    lock_dram_grant_i : in  std_logic;

    iram_single_error_p_o : out std_logic;
    iram_double_error_p_o : out std_logic;

    iram_single_error_bank1_p_o : out std_logic;
    iram_double_error_bank1_p_o : out std_logic;
      
    dram_single_error_p_o : out std_logic;
    dram_double_error_p_o : out std_logic;
    
    irq_i             : in std_logic;
    -- to the Wishbone xbar
    cpu_dwb_o : out t_wishbone_master_out;
    cpu_dwb_i : in  t_wishbone_master_in

    );
end entity urv_core_DTMR;


architecture rtl of urv_core_DTMR is
  component secded_ecc is
    generic (
      g_addr_width : natural := 15
      );
    port (
      clk_i : in std_logic;
      rst_i : in std_logic;

      -- to the processor/bus
      d_i   : in std_logic_vector(31 downto 0);
      a_i   : in std_logic_vector(g_addr_width-1 downto 0);
      we_i  : in std_logic;
      bwe_i : in std_logic_vector (3 downto 0);
      re_i  : in std_logic;

      q_o      : out std_logic_vector(31 downto 0);
      done_r_o : out std_logic;
      done_w_o : out std_logic;

      --to the BRAM
      d_ram_i      : in  std_logic_vector (38 downto 0);
      q_ram_o      : out std_logic_vector (38 downto 0);
      a_ram_o      : out std_logic_vector (g_addr_width-1 downto 0);
      we_ram_o     : out std_logic;
      re_ram_o     : out std_logic;
      valid_ram_i  : in  std_logic;
      lock_req_o   : out std_logic;
      lock_grant_i : in  std_logic;

      single_error_p_o : out std_logic;
      double_error_p_o : out std_logic;
      read_fault_p_o   : out std_logic;
      write_fault_p_o  : out std_logic
      );


  end component;

  component secded_ecc_daec is
    generic (
      g_addr_width : natural := 15
      );
    port (
      clk_i : in std_logic;
      rst_i : in std_logic;

      -- to the processor/bus
      d_i   : in std_logic_vector(31 downto 0);
      a_i   : in std_logic_vector(g_addr_width-1 downto 0);
      we_i  : in std_logic;
      bwe_i : in std_logic_vector (3 downto 0);
      re_i  : in std_logic;

      q_o      : out std_logic_vector(31 downto 0);
      done_r_o : out std_logic;
      done_w_o : out std_logic;

      --to the BRAM
      d_ram_i      : in  std_logic_vector (41 downto 0);
      q_ram_o      : out std_logic_vector (41 downto 0);
      a_ram_o      : out std_logic_vector (g_addr_width-1 downto 0);
      we_ram_o     : out std_logic;
      re_ram_o     : out std_logic;
      valid_ram_i  : in  std_logic;
      lock_req_o   : out std_logic;
      lock_grant_i : in  std_logic;

      single_error_p_o : out std_logic;
      double_error_p_o : out std_logic;
      read_fault_p_o   : out std_logic;
      write_fault_p_o  : out std_logic
      );


  end component;


  component urv_cpu is
    port(
      clk_i : in std_logic;
      rst_i : in std_logic;

      irq_i : in std_logic;

      im_addr_o  : out std_logic_vector(31 downto 0);
      im_data_i  : in  std_logic_vector(31 downto 0);
      im_valid_i : in  std_logic;

      dm_addr_o        : out std_logic_vector(31 downto 0);
      dm_data_s_o      : out std_logic_vector(31 downto 0);
      dm_data_l_i      : in  std_logic_vector(31 downto 0);
      dm_data_select_o : out std_logic_vector(3 downto 0);
      --dm_ready_i       : in  std_logic;
      dm_store_o       : out std_logic;
      dm_load_o        : out std_logic;
      dm_load_done_i   : in  std_logic;
      dm_store_done_i  : in  std_logic
      );
  end component;

component rst_filter is

  
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;
    
    rst_o : out std_logic );
end component;
  

 
  signal im_addr, dm_addr, dram_addr          : std_logic_vector(31 downto 0);
  signal im_data, im_data_cpu, im_data_bank1, dram_data_in, dram_data_out : std_logic_vector(31 downto 0);
  signal im_valid, im_valid_cpu, im_valid_bank1                             : std_logic;

  signal dm_data_s, dm_data_l, dm_data_ram, dm_wb_rdata         : std_logic_vector(31 downto 0);
  signal dm_data_select, dram_bwe                               : std_logic_vector(3 downto 0);
  signal dm_load, dm_store, dm_done, dm_ready, dram_we, dram_re : std_logic;
  signal dram_done_r, dram_done_w                               : std_logic;

  signal dm_is_wishbone, dm_cycle_in_progress, dm_load_done_wb, dm_store_done_wb : std_logic;
  signal dm_select_wb, dm_wb_write                                               : std_logic;
  signal dm_store_done, dm_load_done                                             : std_logic;
  signal rst : std_logic;
  
  attribute syn_hier : string;
attribute syn_hier of rtl: architecture is "fixed";
begin



  filter : rst_filter
  port map (
    clk_i => clk_i,
    rst_i => rst_i,
    rst_o => rst);
    

  secded_iram : secded_ecc
    generic map (
      g_addr_width => 16
      )
    port map (
      clk_i => clk_i,
      rst_i => rst,

      -- to the processor/bus
      d_i   => (others => '0'),
      a_i   => im_addr (15 downto 0),
      we_i  => '0',
      bwe_i => (others => '1'),
      re_i  => '1',

      q_o      => im_data,
      done_r_o => im_valid,
      done_w_o => open,

      --to the BRAM
      d_ram_i     => d_iram_i,
      q_ram_o     => q_iram_o,
      a_ram_o     => a_iram_o,
      we_ram_o    => we_iram_o,
      re_ram_o    => re_iram_o,
      valid_ram_i => valid_iram_i,

      lock_req_o   => lock_iram_req_o,
      lock_grant_i => lock_iram_grant_i,

      single_error_p_o => iram_single_error_p_o,
      double_error_p_o => iram_double_error_p_o,
      read_fault_p_o   => open,
      write_fault_p_o  => open
      );

gen : if g_m2s090 generate
--secded_iram_bank1 : secded_ecc_daec
--    generic map (
--      g_addr_width => 15
--      )
--    port map (
--      clk_i => clk_i,
--      rst_i => rst,
--
--      -- to the processor/bus
--      d_i   => (others => '0'),
--      a_i   => im_addr (14 downto 0),
--      we_i  => '0',
--      bwe_i => (others => '1'),
--      re_i  => im_addr(16),
--
--      q_o      => im_data_bank1,
--      done_r_o => im_valid_bank1,
--      done_w_o => open,
--
--      --to the BRAM
--      d_ram_i     => d_iram_bank1_i,
--      q_ram_o     => q_iram_bank1_o,
--      a_ram_o     => open,
--      we_ram_o    => we_iram_bank1_o,
--      re_ram_o    => re_iram_bank1_o,
--      valid_ram_i => valid_iram_bank1_i,
--
--      lock_req_o   => lock_iram_req_bank1_o,
--      lock_grant_i => lock_iram_grant_bank1_i,
--
--      single_error_p_o => iram_single_error_bank1_p_o,
--      double_error_p_o => iram_double_error_bank1_p_o,
--      read_fault_p_o   => open,
--      write_fault_p_o  => open
--      );


 secded_dram : secded_ecc
    generic map (
      g_addr_width => 16
      )
    port map (
      clk_i => clk_i,
      rst_i => rst,

      -- to the processor/bus
      d_i   => dram_data_in,
      a_i   => dram_addr (15 downto 0),
      we_i  => dram_we and not dram_done_w,
      bwe_i => dram_bwe,
      re_i  => dram_re and not dram_done_r,

      q_o      => dram_data_out,
      done_r_o => dram_done_r,
      done_w_o => dram_done_w,

      --to the BRAM
      d_ram_i     => d_dram_i,
      q_ram_o     => q_dram_o,
      a_ram_o     => a_dram_o,
      we_ram_o    => we_dram_o,
      re_ram_o    => re_dram_o,
      valid_ram_i => valid_dram_i,

      lock_req_o   => lock_dram_req_o,
      lock_grant_i => lock_dram_grant_i,

      single_error_p_o => dram_single_error_p_o,
      double_error_p_o => dram_double_error_p_o,
      read_fault_p_o   => open,
      write_fault_p_o  => open
      );

end generate;

gen_daec : if not g_m2s090 generate

  secded_dram : secded_ecc_daec
    generic map (
      g_addr_width => 15
      )
    port map (
      clk_i => clk_i,
      rst_i => rst,

      -- to the processor/bus
      d_i   => dram_data_in,
      a_i   => dram_addr (14 downto 0),
      we_i  => dram_we and not dram_done_w,
      bwe_i => dram_bwe,
      re_i  => dram_re and not dram_done_r,

      q_o      => dram_data_out,
      done_r_o => dram_done_r,
      done_w_o => dram_done_w,

      --to the BRAM
      d_ram_i     => d_dram_i,
      q_ram_o     => q_dram_o,
      a_ram_o     => a_dram_o,
      we_ram_o    => we_dram_o,
      re_ram_o    => re_dram_o,
      valid_ram_i => valid_dram_i,

      lock_req_o   => lock_dram_req_o,
      lock_grant_i => lock_dram_grant_i,

      single_error_p_o => dram_single_error_p_o,
      double_error_p_o => dram_double_error_p_o,
      read_fault_p_o   => open,
      write_fault_p_o  => open
      );

end generate;

  dm_is_wishbone <= dm_addr(31);

  -- Wishbone bus arbitration / internal RAM access
  process(clk_i)
  begin
    if rising_edge(clk_i) then
      if(rst = '1') then
        cpu_dwb_o.cyc        <= '0';
        dm_cycle_in_progress <= '0';
        dm_load_done_wb      <= '0';
        dm_store_done_wb     <= '0';
        dm_select_wb         <= '0';
        cpu_dwb_o.cyc        <= '0';
        cpu_dwb_o.stb        <= '0';
        dram_we <= '0';
        dram_re <= '0';
        dram_addr <= (others => '0');
      else
        -- signals from uRV data interface to the data-ram are sampled to improve fmax
      if (dram_done_r or dram_done_w) then
          dram_we <= '0';
          dram_re <= '0';
      end if;
      
        if (dm_is_wishbone = '0' and (dm_load = '1' or dm_store = '1')) then
          dram_addr    <= dm_addr;
          dram_data_in <= dm_data_s;
          dram_bwe     <= dm_data_select;
          dram_we      <= dm_store;
          dram_re      <= dm_load;
          dm_select_wb <= '0';
        end if;


        if(dm_cycle_in_progress = '0') then
          dm_select_wb <= '0';
          if(dm_is_wishbone = '1' and (dm_load = '1' or dm_store = '1')) then
            cpu_dwb_o.cyc <= '1';
            cpu_dwb_o.stb <= '1';
            cpu_dwb_o.we  <= dm_store;
            dm_wb_write   <= dm_store;

            cpu_dwb_o.adr <= dm_addr;
            cpu_dwb_o.dat <= dm_data_s;
            cpu_dwb_o.sel <= dm_data_select;


            dm_load_done_wb      <= '0';
            dm_store_done_wb     <= '0';
            dm_cycle_in_progress <= '1';
          else
            dm_store_done_wb     <= '0';
            dm_load_done_wb      <= '0';
            dm_cycle_in_progress <= '0';
          end if;
          
        else
          if(cpu_dwb_i.stall = '0') then
            cpu_dwb_o.stb <= '0';
          end if;

          if(cpu_dwb_i.ack = '1') then
            if(dm_wb_write = '0') then
              dm_wb_rdata     <= cpu_dwb_i.dat;
              dm_load_done_wb <= '1';
            else
              dm_store_done_wb <= '1';
            end if;
            dm_select_wb         <= '1';
            dm_cycle_in_progress <= '0';
            cpu_dwb_o.cyc        <= '0';
          end if;
        end if;
      end if;
    end if;
  end process;


  dm_store_done <= dm_store_done_wb when (dm_select_wb = '1') else dram_done_w;
  dm_load_done  <= dm_load_done_wb  when (dm_select_wb = '1') else dram_done_r;
  dm_data_l     <= dm_wb_rdata      when (dm_select_wb = '1') else dram_data_out;
--
--  im_data_cpu <= im_data when im_addr(16) = '0' else im_data_bank1;
--  im_valid_cpu <= im_valid when im_addr(16) = '0' else im_valid_bank1;
  
  cpu_core : urv_cpu
    port map (
      clk_i            => clk_i,
      rst_i            => rst,
      irq_i            => irq_i,
      im_addr_o        => im_addr,
      im_data_i        => im_data,
      im_valid_i       => im_valid,
      dm_addr_o        => dm_addr,
      dm_data_s_o      => dm_data_s,
      dm_data_l_i      => dm_data_l,
      dm_data_select_o => dm_data_select,
     -- dm_ready_i       => '1',
      dm_store_o       => dm_store,
      dm_load_o        => dm_load,
      dm_load_done_i   => dm_load_done,
      dm_store_done_i  => dm_store_done);


end architecture;