
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use STD.textio.all;


library work;
use work.wishbone_pkg.all;



use work.voter.ALL;
use work.wb_master_voter_pkg.all;
use work.wb_slave_voter_pkg.all;

entity urv_soc is

  port (
    clk_i     : in std_logic;
    clk_ram_i : in std_logic_vector (39 downto 0);
    rst_i     : in std_logic;

    DEVRST_I : in std_logic;


    apb_psel_o      : out std_logic;
	apb_pwr_o		: out std_logic; 
	apb_pen_o 		: out std_logic;
	apb_paddr_o 	: out std_logic_vector(31 downto 0);
	apb_pwdat_o 	: out std_logic_vector(31 downto 0);
	apb_prdat_i		: in std_logic_vector(31 downto 0);
	apb_pready_i	: in std_logic; --
	apb_pslverr_i	: in std_logic; --

 
    uart_tx_o      : out std_logic;


    --TX
    eth_tx_clk_i : in  std_logic;
    eth_txd_o    : out std_logic_vector (3 downto 0);
    eth_txen_o   : out std_logic;
    eth_txerr_o  : out std_logic;

    --RX
    eth_rx_clk_i : in std_logic;
    eth_rxd_i    : in std_logic_vector (3 downto 0);
    eth_rxdv_i   : in std_logic;
    eth_rxerr_i  : in std_logic;


    --MIIM
    eth_mdc_o : out std_logic;
    eth_md_i  : in  std_logic;
    eth_md_o  : out std_logic;
    eth_oe_o  : out std_logic;

    eth_rst_o : out std_logic;

    reboot_o : out std_logic
    );
end entity urv_soc;


architecture rtl of urv_soc is

 
constant g_dram_data_width : integer := 39;
constant g_dram_addr_width : integer := 16;
constant g_m2s090 : boolean := true;

  function f_voter_stdlogic (a : std_logic; b : std_logic; c : std_logic) return std_logic is
  begin
	return  (a and b) or (a and c) or (b and c);
  end f_voter_stdlogic;


  component uart
    port (
      reset_i    : in  std_logic;
      clk_i      : in  std_logic;
      txd_o      : out std_logic;
      tx_data_i  : in  std_logic_vector(7 downto 0);
      tx_wdone_o : out std_logic;
      tx_ready_o : out std_logic;
      tx_req_i   : in  std_logic);
  end component;


  component xwb_uart

    port(
      clk_sys_i : in std_logic;
      rst_i     : in std_logic;

      -- Wishbone
      slave_i : in  t_wishbone_slave_in;
      slave_o : out t_wishbone_slave_out;
      desc_o  : out t_wishbone_device_descriptor;

-- Port for BIT field: 'TX busy' in reg: 'Status Register'
      uart_sr_tx_busy_i  : in  std_logic;
-- Port for MONOSTABLE field: 'TX trigger' in reg: 'Status Register'
      uart_sr_tx_send_o  : out std_logic;
      uart_tdr_tx_data_o : out std_logic_vector(7 downto 0)

      );

  end component;


  component xwb_seu_regs

    port(
      clk_sys_i : in std_logic;
      rst_i     : in std_logic;

      -- Wishbone
      slave_i : in  t_wishbone_slave_in;
      slave_o : out t_wishbone_slave_out;
      desc_o  : out t_wishbone_device_descriptor;

     
    iram_seu_regs_see_se_i : in    std_logic_vector(15 downto 0);
    iram_seu_regs_see_de_i : in    std_logic_vector(15 downto 0);
    dram_seu_regs_see_se_i : in    std_logic_vector(15 downto 0);
    dram_seu_regs_see_de_i : in    std_logic_vector(15 downto 0)
      );

  end component;


  component xwb_wd_reg

    port(
      clk_sys_i : in std_logic;
      rst_i     : in std_logic;

      -- Wishbone
      slave_i            : in  t_wishbone_slave_in;
      slave_o            : out t_wishbone_slave_out;
      desc_o             : out t_wishbone_device_descriptor;
	  -- Port for MONOSTABLE field: 'Heartbeat' in reg: 'Watchdog'
      wd_reg_heartbeat_o : out std_logic
      );

  end component;



  component xwb_ram_adapter

    generic (
      g_addr_width : natural := 15;
      g_data_width :natural := 38;
      g_is_iram : boolean := false;
      g_is_dram : boolean := false

      );
    port (
      clk_i : in std_logic;
      rst_i : in std_logic;
      boot_done_i : in std_logic;

      -- Wishbone
      slave_i : in  t_wishbone_slave_in;
      slave_o : out t_wishbone_slave_out;

      -- ram interface
      d_ram_i      : in  std_logic_vector (g_data_width-1 downto 0);
      q_ram_o      : out std_logic_vector (g_data_width-1 downto 0);
      a_ram_o      : out std_logic_vector (g_addr_width-1 downto 0);
      we_ram_o     : out std_logic;
      re_ram_o     : out std_logic;
      valid_ram_i  : in  std_logic;
      lock_req_o   : out std_logic;
      lock_grant_i : in  std_logic;
      single_error_p_o : out std_logic;
      double_error_p_o : out std_logic

      );
  end component;



  component dpram_generic

    generic (
      g_addr_width : natural := 15;
      g_data_width : natural := 38
      );
    port (
      clk_i : in std_logic;
      rst_i : in std_logic;
      clk_ram_i : in std_logic_vector (39 downto 0);
      da_i  : in std_logic_vector(g_data_width-1 downto 0);
      db_i  : in std_logic_vector(g_data_width-1 downto 0);
      aa_i  : in std_logic_vector(g_addr_width-1 downto 0);
      ab_i  : in std_logic_vector(g_addr_width-1 downto 0);
      wa_i  : in std_logic_vector (3 downto 1);
      wb_i  : in std_logic;
      ra_i  : in std_logic;
      rb_i  : in std_logic;

      qa_o : out std_logic_vector(g_data_width-1 downto 0);
      qb_o : out std_logic_vector(g_data_width-1 downto 0);

      lock_a_req_i   : in  std_logic;
      lock_b_req_i   : in  std_logic;
      lock_a_grant_o : out std_logic;
      lock_b_grant_o : out std_logic;

      valid_a_o : out std_logic;
      valid_b_o : out std_logic

       
      );
  end component;


 

        component ram_voter 
          generic (
            g_addr_width : natural := 15;
            g_data_width : natural := 38

            );
          port (


            da_vot_i : in t_voter_stdvector;
            aa_vot_i : in t_voter_stdvector;
            wa_vot_i : in t_voter_stdlogic;
            ra_vot_i : in t_voter_stdlogic;

            lock_a_req_vot_i : in t_voter_stdlogic;

            da_o         : out std_logic_vector(g_data_width-1 downto 0);
            aa_o         : out std_logic_vector(g_addr_width-1 downto 0);
            wa_o         : out std_logic;
            ra_o         : out std_logic;
            lock_a_req_o : out std_logic
            );


        end component;

      


component seu_counters is
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;
    
    event_se_iram : in std_logic;
    event_de_iram : in std_logic;
    event_se_dram : in std_logic;
    event_de_dram : in std_logic;    
    
        
    se_dram_o : out std_logic_vector (15 downto 0);
    de_dram_o : out std_logic_vector (15 downto 0);
    
    se_iram_o : out std_logic_vector (15 downto 0);
    de_iram_o : out std_logic_vector (15 downto 0)
    
    );

end component;


        constant c_SLAVE_NUM  : natural := 7;
		-- only one master to reduce LUT
		-- tested in radiation also with 2 masters, no problems 
        constant c_MASTER_NUM : natural := 1;

        constant c_SLAVE_WD   : integer := 0;
        constant c_SLAVE_UART : integer := 1;
        constant c_SLAVE_SEC  : integer := 2;
        constant c_SLAVE_ETH  : integer := 3;
        constant c_SLAVE_DRAM : integer := 4;
        constant c_SLAVE_SUPERVISOR : integer := 5;
		constant c_SLAVE_ENVM : integer := 6;

        constant c_CNX_ADDRESS : t_wishbone_address_array(c_SLAVE_NUM-1 downto 0) := (
          c_SLAVE_WD            => x"81000000",
          c_SLAVE_UART          => x"82000000",
          c_SLAVE_SEC           => x"83000000",
          c_SLAVE_ETH           => x"84000000",
          c_SLAVE_DRAM          => x"85000000",
          c_SLAVE_SUPERVISOR    => x"86000000",
		  c_SLAVE_ENVM			=> x"87000000"
          );

        constant c_CNX_MASK : t_wishbone_address_array(c_SLAVE_NUM-1 downto 0) := (
          c_SLAVE_WD            => x"ff000000",
          c_SLAVE_UART          => x"ff000000",
          c_SLAVE_SEC           => x"ff000000",
          c_SLAVE_ETH           => x"FF000000",
          c_SLAVE_DRAM          => x"FF000000",
          c_SLAVE_SUPERVISOR    => x"ff000000",
		  c_SLAVE_ENVM			=> x"FF000000"

          );

        signal cnx_master_in  : t_wishbone_master_in_array(c_SLAVE_NUM-1 downto 0);
        signal cnx_master_out : t_wishbone_master_out_array(c_SLAVE_NUM-1 downto 0);
        signal cnx_slave_in   : t_wishbone_slave_in_array(c_MASTER_NUM-1 downto 0);
        signal cnx_slave_out  : t_wishbone_slave_out_array(c_MASTER_NUM-1 downto 0);


        signal cpu_dwb_out_voted : t_wishbone_master_out;
        signal cpu_dwb_out : t_voter_wb_master; 
        signal cpu_dwb_in  : t_wishbone_master_in;
        
        signal envm_wb_iram_in: t_wishbone_slave_in;
        signal envm_wb_iram_out : t_wishbone_slave_out;

        signal eth_m_wb_out  : t_wishbone_master_out;
        signal eth_m_wb_in : t_wishbone_master_in;

        signal supervisor_wb_out : t_voter_wb_slave;
        signal rst_cpu_sv, reboot_sv, boot_done_sv : std_logic_vector (3 downto 1);
        signal rst_cpu_sv_voted, reboot_sv_voted : std_logic;
		signal boot_done : std_logic;

        signal cpu_rst, cpu_rst_d : std_logic;

   

        signal first_de : std_logic;

        signal de_counter, se_counter             : unsigned (15 downto 0);
        
        type fsm_uart_states is (idle, wait_write, wait_lf);  
        signal fsm_uart : fsm_uart_states := idle;

        signal uart_ready, uart_ready_wb, dm_uart_write_done          : std_logic;
        signal uart_tx_data, uart_tx_data_wb : std_logic_vector (7 downto 0);
        signal dm_counter_read_done, dm_uart_read_done : std_logic;

        signal rst_cpu, rst_cpu_boot, rst_cpu_wd   : std_logic;
        signal wd_counter            : unsigned (28 downto 0);
        signal dm_counter_write_done : std_logic;

        signal wd_heartbeat : std_logic;

        signal tx_req, tx_req_wb : std_logic;


        signal q_cpu_iram    : t_voter_stdvector (open)(38 downto 0);
        signal q_cpu_dram : t_voter_stdvector(open)(g_dram_data_width-1 downto 0);
        signal a_cpu_iram                             : t_voter_stdvector (open)(15 downto 0);
        signal a_cpu_dram                             : t_voter_stdvector (open)(g_dram_addr_width-1 downto 0);
        signal we_cpu_iram, we_cpu_dram : std_logic_vector (3 downto 1);
        signal re_cpu_iram, re_cpu_dram : t_voter_stdlogic;
        signal lock_cpu_dram_req, lock_cpu_iram_req : t_voter_stdlogic;
signal selector : std_logic;
        signal d_cpu_iram, d_wb_iram, d_cpu_iram_seu              : std_logic_vector (38 downto 0);
        signal d_cpu_dram, d_wb_dram : std_logic_vector (g_dram_data_width-1 downto 0);
        signal q_cpu_iram_voted, q_wb_iram : std_logic_vector (38 downto 0);
signal q_cpu_dram_voted, q_wb_dram : std_logic_vector (g_dram_data_width-1 downto 0);

        signal a_cpu_iram_voted, a_wb_iram                                                  : std_logic_vector (15 downto 0);
        signal a_cpu_dram_voted, a_wb_dram                                                  : std_logic_vector (g_dram_addr_width-1 downto 0);
        signal re_cpu_iram_voted, re_cpu_dram_voted, re_wb_dram, re_wb_iram                 : std_logic;
        signal we_cpu_iram_voted, we_cpu_dram_voted, we_wb_dram, we_wb_iram                 : std_logic;
        signal lock_cpu_dram_req_voted, lock_wb_dram_req, lock_cpu_dram_grant, lock_wb_dram_grant : std_logic;
        signal lock_cpu_iram_req_voted, lock_wb_iram_req, lock_wb_iram_grant, lock_cpu_iram_grant  : std_logic;
        signal valid_cpu_iram, valid_cpu_dram, valid_wb_iram, valid_wb_dram                 : std_logic;

        signal se_iram, se_dram : std_logic_vector (1 to 3);
        signal de_iram, de_dram : std_logic_vector (1 to 3);
        
        signal se_iram_voted, se_dram_voted, de_iram_voted, de_dram_voted : std_logic;
        signal se_iram_scrubber, de_iram_scrubber : std_logic;
        signal se_dram_scrubber, de_dram_scrubber : std_logic;
        
        signal reboot_board, reboot_board_f : std_logic;
      
        signal irq_eth : std_logic;
        signal uart_debug_ready, tx_debug_req : std_logic;
        signal uart_debug_data : std_logic_vector (7 downto 0);
        signal se_iram_counts, de_iram_counts, se_dram_counts, de_dram_counts : std_logic_vector (15 downto 0);

     
        signal first_time : std_logic;

        attribute syn_radhardlevel        : string;
        attribute syn_radhardlevel of  rtl : architecture is "tmr";
        
        begin

          rst_cpu        <= rst_cpu_boot;
          
          reboot_board <=  rst_cpu_wd or reboot_sv_voted; --(not rst_cpu and (de_iram_voted or de_dram_voted or de_iram_scrubber or de_dram_scrubber)) ;
          reboot_o <= reboot_board;

      

          iram : dpram_generic
            generic map (
              g_addr_width => 16,
              g_data_width => 39)
            port map (
              clk_i => clk_i,
              clk_ram_i => clk_ram_i, 
              rst_i => rst_i,
              da_i  => q_cpu_iram_voted,
              db_i  => q_wb_iram,
              aa_i  => a_cpu_iram_voted,
              ab_i  => a_wb_iram,
              wa_i  => we_cpu_iram,--we_cpu_iram_voted,
              wb_i  => we_wb_iram,
              ra_i  => re_cpu_iram_voted,
              rb_i  => re_wb_iram,

              lock_a_req_i   => lock_cpu_iram_req_voted,
              lock_b_req_i   => lock_wb_iram_req,
              lock_a_grant_o => lock_cpu_iram_grant,
              lock_b_grant_o => lock_wb_iram_grant,

              qa_o      => d_cpu_iram,
              qb_o      => d_wb_iram,
              valid_a_o => valid_cpu_iram,
              valid_b_o => valid_wb_iram
             
              );



          iram_voter : ram_voter
            generic map(
              g_addr_width => 16,
              g_data_width => 39
              )
            port map (
              da_vot_i => q_cpu_iram,
              aa_vot_i => a_cpu_iram,
              wa_vot_i => (others => '0'), --we_cpu_iram,
              ra_vot_i => re_cpu_iram,

              lock_a_req_vot_i => lock_cpu_iram_req,
              lock_a_req_o     => lock_cpu_iram_req_voted,

              da_o => q_cpu_iram_voted,
              aa_o => a_cpu_iram_voted,
             -- wa_o => we_cpu_iram_voted,
              ra_o => re_cpu_iram_voted
            
              );

  
          dram : dpram_generic
            generic map (
              g_addr_width => g_dram_addr_width,
              g_data_width => g_dram_data_width)
            port map (
              clk_i => clk_i,
              clk_ram_i => clk_ram_i,
              rst_i => rst_i,
              da_i  => q_cpu_dram_voted,
              db_i  => q_wb_dram,
              aa_i  => a_cpu_dram_voted,
              ab_i  => a_wb_dram,
              wa_i  => we_cpu_dram,--we_cpu_dram_voted,
              wb_i  => we_wb_dram,
              ra_i  => re_cpu_dram_voted,
              rb_i  => re_wb_dram,


              lock_a_req_i   => lock_cpu_dram_req_voted,
              lock_b_req_i   => lock_wb_dram_req,
              lock_a_grant_o => lock_cpu_dram_grant,
              lock_b_grant_o => lock_wb_dram_grant,

              qa_o      => d_cpu_dram,
              qb_o      => d_wb_dram,
              valid_a_o => valid_cpu_dram,
              valid_b_o => valid_wb_dram

              
              );

 
        
          dram_voter : ram_voter
            generic map(
              g_addr_width => g_dram_addr_width,
              g_data_width => g_dram_data_width)
            port map (
              da_vot_i => q_cpu_dram,
              aa_vot_i => a_cpu_dram,
              wa_vot_i => (others => '0'), --we_cpu_dram,
              ra_vot_i => re_cpu_dram,

              lock_a_req_vot_i => lock_cpu_dram_req,
              lock_a_req_o     => lock_cpu_dram_req_voted,

              da_o => q_cpu_dram_voted,
              aa_o => a_cpu_dram_voted,
              --wa_o => we_cpu_dram_voted,
              ra_o => re_cpu_dram_voted
              );
          

            

        gen_cores :  for k in 1 to 3 generate
      
              urv_core : entity work.urv_core_DTMR

                      generic map(
                  g_m2s090 => g_m2s090,
                  g_dram_data_width => g_dram_data_width,
                  g_dram_addr_width => g_dram_addr_width
                 )
                port map (
                  clk_i => clk_i,
                  rst_i => rst_cpu or rst_cpu_sv (k), --(1)rst_cpu_sv(k), --rst_cpu,

                  -- to the IRAM primitive
                  d_iram_i          => d_cpu_iram,
                  q_iram_o          => q_cpu_iram(k),
                  a_iram_o          => a_cpu_iram(k),
                  we_iram_o         => we_cpu_iram(k),
                  re_iram_o         => re_cpu_iram(k),
                  valid_iram_i      => valid_cpu_iram,
                  lock_iram_req_o   => lock_cpu_iram_req(k),
                  lock_iram_grant_i => lock_cpu_iram_grant,
                          


                  -- to the DRAM primitive
                  d_dram_i          => d_cpu_dram,
                  q_dram_o          => q_cpu_dram(k),
                  a_dram_o          => a_cpu_dram(k),
                  we_dram_o         => we_cpu_dram(k),
                  re_dram_o         => re_cpu_dram(k),
                  valid_dram_i      => valid_cpu_dram,
                  lock_dram_req_o   => lock_cpu_dram_req(k),
                  lock_dram_grant_i => lock_cpu_dram_grant,

                  iram_single_error_p_o => se_iram(k),
                  iram_double_error_p_o => de_iram(k),
      
                  dram_single_error_p_o => se_dram(k),
                  dram_double_error_p_o => de_dram(k),
                  irq_i => irq_eth,
                  -- to the Wishbone xbar
                  cpu_dwb_o => cpu_dwb_out(k),
                  cpu_dwb_i => cpu_dwb_in

                  );

              supervisor :  entity work.urv_supervisor 
                generic map (
                  g_cpu_id => k)
                port map(
                  clk_i => clk_i,
                  rst_i => rst_i,

                  slave_i => cnx_master_out(c_SLAVE_SUPERVISOR),
                  slave_o => supervisor_wb_out (k),

                  pc_i    => a_cpu_iram,
                  stall_i => ('0', '0', '0'),
				  boot_done_o => boot_done_sv (k),
                  rst_cpu_o => rst_cpu_sv (k),
                  reboot_o  => reboot_sv (k)

                  );
             
            
             end generate;

              wb_cpu_voter : entity work.wb_master_voter
                port map (
                m_wb_i => cpu_dwb_out,
                m_wb_o => cpu_dwb_out_voted
                );

               wb_supervisor_voter : entity work.wb_slave_voter
                port map (
                s_wb_i => supervisor_wb_out,
                s_wb_o => cnx_master_in(c_SLAVE_SUPERVISOR)
                );

            reboot_sv_voted <= f_voter_stdlogic (reboot_sv(1), reboot_sv(2), reboot_sv(3));
			boot_done <= f_voter_stdlogic (boot_done_sv (1), boot_done_sv(2),boot_done_sv(3));  
              
            se_iram_voted <= f_voter_stdlogic(se_iram (1), se_iram(2), se_iram(3)); 
            se_dram_voted <= f_voter_stdlogic(se_dram (1), se_dram(2), se_dram(3));
            de_iram_voted <= f_voter_stdlogic(de_iram (1), de_iram(2), de_iram(3));
            de_dram_voted <= f_voter_stdlogic(de_dram (1), de_dram(2), de_dram(3));

              
        envm_wb : entity work.wb_envm
        port map (
            clk_i => clk_i,
            rst_i => rst_i,
			rst_cpu_o => rst_cpu_boot,
			
			apb_psel_o => apb_psel_o,
			apb_pwr_o => apb_pwr_o,	
			apb_pen_o => apb_pen_o,
			apb_paddr_o => apb_paddr_o,
			apb_pwdat_o => apb_pwdat_o,
			apb_prdat_i	=> apb_prdat_i,
			apb_pready_i => apb_pready_i,
			
			master_i => envm_wb_iram_out,
			master_o => envm_wb_iram_in,
			
			slave_i => cnx_master_out (c_SLAVE_ENVM),
			slave_o => cnx_master_in (c_SLAVE_ENVM)
			);
	
      
       

          xwb_iram : xwb_ram_adapter
            generic map (
              g_addr_width => 16,
              g_data_width => 39,
              g_is_iram => true,
              g_is_dram => false
              )
            port map(
              clk_i => clk_i,
              rst_i => rst_i,
              boot_done_i => boot_done,

              -- Wishbone
              slave_i => envm_wb_iram_in,
              slave_o => envm_wb_iram_out,

              -- ram interface
              d_ram_i      => d_wb_iram,
              q_ram_o      => q_wb_iram,
              a_ram_o      => a_wb_iram,
              we_ram_o     => we_wb_iram,
              re_ram_o     => re_wb_iram,
              valid_ram_i  => valid_wb_iram,
              lock_req_o   => lock_wb_iram_req,
              lock_grant_i => lock_wb_iram_grant,
              single_error_p_o => se_iram_scrubber,
              double_error_p_o => de_iram_scrubber
              );
            
            



              

          xwb_dram : xwb_ram_adapter
            generic map (
              g_addr_width => g_dram_addr_width,
              g_data_width => g_dram_data_width,
              g_is_iram => false,
              g_is_dram => true
              )
            port map(
              clk_i => clk_i,
              rst_i => rst_i,
              boot_done_i => boot_done,

              -- Wishbone
              slave_i => eth_m_wb_out, --cnx_master_out(c_SLAVE_DRAM),
              slave_o => eth_m_wb_in,--cnx_master_in(c_SLAVE_DRAM),

              -- ram interface
              d_ram_i      => d_wb_dram,
              q_ram_o      => q_wb_dram,
              a_ram_o      => a_wb_dram,
              we_ram_o     => we_wb_dram,
              re_ram_o     => re_wb_dram,
              valid_ram_i  => valid_wb_dram,
              lock_req_o   => lock_wb_dram_req,
              lock_grant_i => lock_wb_dram_grant,
              single_error_p_o => se_dram_scrubber,
              double_error_p_o => de_dram_scrubber
              );




          
          U_Local_Interconnect : xwb_crossbar
            generic map (
              g_NUM_MASTERS => 1,
              g_NUM_SLAVES  => c_SLAVE_NUM,
              g_REGISTERED  => true,
              g_ADDRESS     => c_CNX_ADDRESS,
              g_MASK        => c_CNX_MASK)
            port map (
              clk_sys_i => clk_i,
              rst_n_i   => not rst_i,
              slave_i   => cnx_slave_in,   --cpu_dwb_out,
              slave_o   => cnx_slave_out,  --cpu_dwb_in,
              master_i  => cnx_master_in,
              master_o  => cnx_master_out);

          cnx_slave_in(0) <= cpu_dwb_out_voted;
          cpu_dwb_in      <= cnx_slave_out(0);


mac : entity work.ueth
port map (
  
    clk_i  => clk_i,
    rst_i  => rst_i,

              --TX
              phy0_tx_clk_i => eth_tx_clk_i,
              phy0_txd_o    => eth_txd_o,
              phy0_txen_o   => eth_txen_o,
           --   phy0_txerr_o  => eth_txerr_o,

              --RX
              phy0_rx_clk_i => eth_rx_clk_i,
              phy0_rxd_i    => eth_rxd_i,
              phy0_rxdv_i   => eth_rxdv_i,
             phy0_rxerr_i  => eth_rxerr_i,
  
                --TX
              phy1_tx_clk_i => eth_tx_clk_i,
              phy1_txd_o    => open,
              phy1_txen_o   => open,
            --  phy1_txerr_o  => open,

              --RX
              phy1_rx_clk_i => eth_rx_clk_i,
              phy1_rxd_i    => (others =>'0'),
              phy1_rxdv_i   => '0',
             phy1_rxerr_i  => '0',
   

              --MIIM
              phy_mdc_o => eth_mdc_o,
              phy_md_i  => eth_md_i,
              phy_md_o  => eth_md_o,
              phy_oe_o  => eth_oe_o,
                
              phy_rst_o => eth_rst_o,
              irq_o => irq_eth,
              -- Wishbone regs
              slave_i => cnx_master_out(c_SLAVE_ETH),
              slave_o => cnx_master_in(c_SLAVE_ETH),

              -- Wishbone DMA
              master_i => eth_m_wb_in,
              master_o => eth_m_wb_out

);




          
         U_seu_regs : xwb_seu_regs
            port map(
              clk_sys_i => clk_i,
              rst_i     => rst_i,

              -- Wishbone
              slave_i => cnx_master_out(c_SLAVE_SEC),
              slave_o => cnx_master_in(c_SLAVE_SEC),

              iram_seu_regs_see_se_i => se_iram_counts,
              iram_seu_regs_see_de_i => de_iram_counts,
              dram_seu_regs_see_se_i => se_dram_counts,
              dram_seu_regs_see_de_i => de_dram_counts
              );
              
              inst_seu_counters:  seu_counters 

  port map(
    clk_i => clk_i,
    rst_i => rst_i,
    
    event_se_iram => se_iram_voted or se_iram_scrubber,
    event_de_iram => de_iram_voted or de_iram_scrubber,
     
    event_se_dram => se_dram_voted or se_dram_scrubber,
    event_de_dram => de_dram_voted or de_dram_scrubber,
       
    
    se_iram_o => se_iram_counts,
    de_iram_o => de_iram_counts,
    se_dram_o => se_dram_counts,
    de_dram_o => de_dram_counts 
    
        
   
    );



          U_wd_regs : xwb_wd_reg
            port map(
              clk_sys_i => clk_i,
              rst_i     => rst_i,

              -- Wishbone
              slave_i => cnx_master_out(c_SLAVE_WD),
              slave_o => cnx_master_in(c_SLAVE_WD),

              wd_reg_heartbeat_o => wd_heartbeat
              );




          wd : process (clk_i)
          begin
            if rst_i = '1' then
              rst_cpu_wd <= '0';
              wd_counter <= (others => '0');
            elsif rising_edge(clk_i) then
              rst_cpu_wd <= '0';
        
              wd_counter <= wd_counter + 1;
            
              if (wd_heartbeat = '1') then
                wd_counter <= (others => '0');
              end if;
              if (wd_counter = (wd_counter'range => '1')) then
                rst_cpu_wd <= '1';
              end if;
            end if;
          end process;



          
          U_UART_regs : xwb_uart

            port map (
              clk_sys_i          => clk_i,
              rst_i              => rst_i,
              slave_i            => cnx_master_out(c_SLAVE_UART),
              slave_o            => cnx_master_in(c_SLAVE_UART),
              uart_sr_tx_busy_i  => uart_ready_wb,
              uart_sr_tx_send_o  => tx_req_wb,
              uart_tdr_tx_data_o => uart_tx_data_wb
              );

          uart_tx : uart
            port map (
              reset_i    => devrst_i,
              clk_i      => clk_i,
              txd_o      => uart_tx_o,
              tx_data_i  => uart_tx_data,
              tx_wdone_o => dm_uart_write_done,
              tx_ready_o => uart_ready,
              tx_req_i   => tx_req);


              process (clk_i)
              begin
                if devrst_i = '1' then
                  first_time <= '1';
                  uart_ready_wb <= '0';
                  fsm_uart <= idle;
                  tx_req <= '0';
                  first_de <= '1';
                elsif rising_edge (clk_i) then
                    if (rst_i) then
                      first_de <= '1';
                      
                  end if;
                    case (fsm_uart) is
                      when idle =>
                          uart_ready_wb <= uart_ready;
                          tx_req <= tx_req_wb;
                          uart_tx_data <= uart_tx_data_wb;

                          if (first_de and (de_iram_voted or de_dram_voted or de_iram_scrubber or de_iram_scrubber)) then
                            fsm_uart <= wait_write;
                            uart_tx_data <= x"23"; -- 23#
                            tx_req <= '1';
                            first_de <= '0';
                          end if;
                          if (reboot_sv_voted = '1') then
                            fsm_uart <= wait_write;
                            uart_tx_data <= x"24"; -- $
                            tx_req <= '1';
                          end if;
                          if (rst_i = '1') then
                          --  fsm_uart <= wait_write;
                          --  uart_tx_data <= x"25"; -- %
                          --  tx_req <= '1';
                          end if;
                          if (first_time = '1') then
                            first_time <= '0';
                            uart_tx_data <= x"40"; -- @
                            tx_req <= '1';
                            fsm_uart <= wait_write;
                          end if;
                          
                          
                     when wait_write =>
                        if (uart_ready = '1') then
                          tx_req <= '1';
                          uart_tx_data <= x"0A"; 
                          fsm_uart <= wait_lf;
                        end if;
                        
                   when wait_lf  =>
                        if (uart_ready = '1') then
                          tx_req <= '0';
                          fsm_uart <= idle;
                        end if;               
                        when others => fsm_uart <= idle;
                    end case;
                  end if;
                end process;

           


                               -- synthesis translate_off
                  

                  process 
                  begin
                    loop
                      wait until rising_edge(clk_i);
                        if (tx_req = '1') then
                            --std.textio.write(std.textio.output, to_string(uart_tx_data));
                            

                            WRITE (output, (1 => character'val (to_integer(unsigned(uart_tx_data)))));
                    end if;
                  
                end loop;
                    end process;
                 -- synthesis translate_on   

    
        
        

        end architecture;