library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;

--library smartfusion2;
--use smartfusion2.all;

use work.voter.all;
use work.wb_voter_pkg.all;



entity urv_supervisor is
generic  (
  g_cpu_id : natural );
  port (
    clk_i : in std_logic;
    rst_i : in std_logic;

    slave_i : in  t_wishbone_slave_in;
    slave_o : out t_wishbone_slave_out;
    
    pc_i    : in t_voter_stdvector (open)(15 downto 0);
    stall_i : in t_voter_stdlogic;

    rst_cpu_o : out std_logic;
    reboot_o  : out std_logic;
    boot_done_o : out std_logic

    );

end entity urv_supervisor;

architecture rtl of urv_supervisor is
 
  signal rst_cpu : std_logic;
  signal pc_cpu1_ok                       : std_logic;
  signal pc_cpu2_ok                      : std_logic;
    signal pc_cpu3_ok                      : std_logic;
    signal panic, panic_d : std_logic;

  signal state_cpu_1, state_cpu_2, state_cpu_3 : std_logic;
  signal state_cpu : std_logic_vector (3 downto 1);  
    signal release_cpu : std_logic;

  type fsm_states is (boot, boot_cnt, voting, lock_step, fatal, fatal_2);
  constant recovery_dunc                 : std_logic_vector (15 downto 0) := x"0000";
  signal fsm                             : fsm_states                     := voting;

  signal start_recovery : std_logic;
  signal count : unsigned (2 downto 0);
  
  
attribute syn_encoding : string; 
attribute syn_encoding of fsm : signal is "safe,onehot";

  attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";
begin

  rst_cpu_o <= rst_cpu;



  pc_cpu1_ok <= '1' when ((pc_i(1) = pc_i(2)) or (pc_i(1) = pc_i(3))) else '0';
  pc_cpu2_ok <= '1' when ((pc_i(2) = pc_i(1)) or (pc_i(2) = pc_i(3))) else '0';
  pc_cpu3_ok <= '1' when ((pc_i(3) = pc_i(2)) or (pc_i(3) = pc_i(1))) else '0';

  panic <= not (pc_cpu1_ok or pc_cpu2_ok or pc_cpu3_ok);

  state_cpu (1) <= state_cpu_1;
  state_cpu (2) <= state_cpu_2;
  state_cpu (3) <= state_cpu_3;
  
  process (clk_i)
  begin
    if rising_edge (clk_i) then
      if (rst_cpu) then
        panic_d <= '0';
      else
      panic_d      <= panic;

    end if;
    end if;
  end process;

  process (clk_i)
  begin
    if rising_edge (clk_i) then
      if rst_i = '1' then
        fsm       <= boot;
        reboot_o  <= '0';
        rst_cpu <= '1';
        count <= (others => '0');
      else
        case fsm is
          when boot =>
            state_cpu_1 <= '1';
            state_cpu_2 <= '1';
            state_cpu_3 <= '1';
            rst_cpu <= '1';
            reboot_o  <= '0';
            fsm <= boot_cnt;
            count <= (others => '0');
        when boot_cnt =>
            count <= count + 1;
            if (count (2)) then
            fsm <= voting;
          end if;
          when voting =>
            rst_cpu <= '0';
            if (pc_cpu1_ok = '0') then
              fsm       <= lock_step;
              state_cpu_1 <= '0';
            end if;
            if (pc_cpu2_ok = '0') then
             fsm       <=  lock_step;
             state_cpu_2 <= '0';
            end if;
            if (pc_cpu3_ok = '0') then
              fsm       <= lock_step;
              state_cpu_3 <= '0';
            end if;
            if (panic_d = '1' and panic = '1') then
              fsm <= fatal;
            end if;
      
        
          when lock_step =>
             rst_cpu <= not state_cpu(g_cpu_id);
             
             if (start_recovery = '1') then
               fsm <= boot;
              end if;
               
             if (panic_d = '1' and panic = '1') then
              fsm <= fatal;
            end if;           
          when fatal =>
            rst_cpu <= '1';
            state_cpu_1 <= '0';
            state_cpu_2 <= '0';
            state_cpu_3 <= '0';
            reboot_o <= '1';
            fsm <= fatal_2;
          when fatal_2 =>
            fsm <= boot;
          when others =>
            fsm <= boot;
        end case;
      end if;
    end if;
  end process;
    

  xwb_reg :  entity work.xwb_supervisor_reg 
  port map (
    rst_i  => rst_i,
    clk_sys_i  => clk_i,

    -- Wishbone regs
    slave_i => slave_i,
    slave_o => slave_o,
    start_recovery_o => start_recovery,
    cpu_status_i  => state_cpu_3 & state_cpu_2 & state_cpu_1,
    boot_done_o => boot_done_o

  );
    

    

end architecture;