library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--library synplify;
--use synplify.attributes.all;
--use synplify.all;

--library smartfusion2;
--use smartfusion2.all;


entity wb_apb_adapter is

 port (
    clk_i 			: in std_logic;
    rst_i   		: in std_logic;
	
    arm_psel_i      : in std_logic;
	arm_pwr_i 		: in std_logic; 
	arm_pen_i 		: in std_logic;
	arm_paddr_i 	: in std_logic_vector(31 downto 0);
	arm_pwdat_i 	: in std_logic_vector(31 downto 0);
	arm_prdat_o		: out std_logic_vector(31 downto 0);
	arm_pready_o	: out std_logic; --

	
	--wishbone master
	m_wb_adr_o		: out std_logic_vector (31 downto 0);
	m_wb_sel_o		: out std_logic_vector (3 downto 0);
	m_wb_we_o		: out std_logic;
	m_wb_dat_o		: out std_logic_vector (31 downto 0);
	m_wb_dat_i		: in std_logic_vector (31 downto 0);
	m_wb_cyc_o		: out std_logic; 
	m_wb_stb_o		: out std_logic;
	m_wb_ack_i		: in std_logic
 

    );
end entity wb_apb_adapter;


architecture rtl of wb_apb_adapter is

signal addr_scrubber : unsigned (12 downto 0);
signal arm_pready : std_logic;

type fsm_states is (idle, process_arm_apb);
signal fsm : fsm_states := idle;


attribute syn_encoding : string; 
attribute syn_encoding of fsm : signal is "safe,onehot";

  attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";


begin



m_wb_sel_o <= (others=>'1');
arm_pready_o <= arm_pready;

process (clk_i)
begin
if rst_i = '1' then
fsm <= idle;
arm_pready <= '0';
addr_scrubber <= (others => '0');
m_wb_we_o <= '0';
m_wb_cyc_o <= '0';
m_wb_stb_o <= '0';
elsif rising_edge(clk_i) then

	case (fsm) is
		when idle =>
				m_wb_cyc_o <= '0';
				m_wb_stb_o <= '0';
				arm_pready <= '0';
			if (arm_psel_i = '1' and arm_pen_i = '1' and arm_pready = '0') then
				m_wb_cyc_o <= '1';
				m_wb_stb_o <= '1';
        m_wb_we_o <= arm_pwr_i;
        --temp
 m_wb_adr_o <= x"8500" & arm_paddr_i (15 downto 0);
   m_wb_dat_o <= arm_pwdat_i;           
-- end temp
				fsm <= process_arm_apb;
			end if;
		when process_arm_apb =>
			arm_pready <= m_wb_ack_i;

arm_prdat_o <= m_wb_dat_i; 
			if (m_wb_ack_i = '1') then	
				fsm <= idle;
				m_wb_cyc_o <= '0';
				m_wb_stb_o <= '0';
			end if;

		when others =>
			fsm <= idle;
	end case;
end if;
end process;
				
		
			


end architecture;