
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
--library synplify;
--use synplify.attributes.all;
--use synplify.all;

library work;
use work.wishbone_pkg.all;
--library smartfusion2;
--use smartfusion2.all;


entity wb_ram_scrubber is

 port (
    clk_i 			: in std_logic;
    rst_i   		: in std_logic;
	cpu_status_i	: in std_logic;
	    -- Wishbone
    slave_i : in  t_wishbone_slave_in;
    slave_o : out t_wishbone_slave_out;

	-- ram interface
	addr_o : out std_logic_vector(14 downto 0);
	data_o : out std_logic_vector(31 downto 0);
	data_i : in std_logic_vector(31 downto 0);
	wen_o : out std_logic;
	ren_o : out std_logic;
	done_i : in std_logic

    );
end entity wb_ram_scrubber;


architecture rtl of wb_ram_scrubber is

signal addr_scrubber : unsigned (14 downto 0);
signal pwr, wr : std_logic;

type fsm_states is (idle, process_wb, scrubber_read, process_wb_partial_write, process_wb_write);
signal fsm : fsm_states := idle;


attribute syn_encoding : string; 
attribute syn_encoding of fsm : signal is "safe,onehot";

--attribute syn_radhardlevel : string;
--attribute syn_radhardlevel of pwr : signal is "tmr";
--attribute syn_radhardlevel of wr : signal is "tmr";


attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";

signal ack: std_logic;

signal data_read : std_logic_vector (31 downto 0);

function f_mask_word (mask : std_logic_vector (3 downto 0); d1 : std_logic_vector (31 downto 0); d2 : std_logic_vector (31 downto 0)) return std_logic_vector is
variable masked_word : std_logic_vector (31 downto 0);
begin
for i in 1 to 4 loop
if (mask(i-1) = '0') then
	masked_word (i*8-1 downto (i-1)*8) := d2(i*8-1 downto (i-1)*8);
else
	masked_word (i*8-1 downto (i-1)*8) := d1(i*8-1 downto (i-1)*8);	
end if;
end loop;
return masked_word;
end f_mask_word;

begin

slave_o.ack <= ack;

--slave_o.stall <= '0' when slave_i.cyc = '0' else not ack;
slave_o.stall <= (not ack) and slave_i.cyc and slave_i.stb;

process (clk_i)
begin
if rst_i = '1' then
fsm <= idle;
addr_scrubber <= (others => '0');
wen_o <= '0';
ren_o <= '0';
ack <= '0';
elsif rising_edge(clk_i) then

	case (fsm) is
		when idle =>
			wen_o <= '0';
			ren_o <= '0';
			ack <= '0';
			if (slave_i.stb = '1' and slave_i.cyc = '1' and ack = '0') then
				fsm <= process_wb;
                if (slave_i.we = '1' and slave_i.sel = "1111") then
                    wen_o <= '1';
                    ren_o <= '0';
                    data_o <= slave_i.dat;
                else
                    ren_o <= '1';
                    wen_o <= '0';
                end if;
                addr_o <= slave_i.adr (14 downto 0);

			end if;
		when process_wb =>
            wen_o <= '0';
			ren_o <= '0';
			slave_o.dat <= data_i;
			
			if (done_i = '1') then
                if (slave_i.we = '1' and slave_i.sel /= "1111") then
               fsm <= process_wb_partial_write;
               data_read <= data_i;
                else
                    fsm <= idle;
                    ack <= '1';
              end if;
			end if;
        when process_wb_partial_write =>
            data_o <= f_mask_word(slave_i.sel, slave_i.dat, data_read);
            wen_o <= '1';
            ren_o <= '0';
            fsm <= process_wb_write;
        when process_wb_write =>
            wen_o <= '0';
			ren_o <= '0';
			if (done_i = '1') then
                fsm <= idle;
                ack <= '1';
            end if;



		when scrubber_read =>
			ren_o <= '0';
			if (done_i = '1') then	
				fsm <= idle;
				addr_scrubber <= addr_scrubber + 4;
			end if;
		when others =>
			fsm <= idle;
	end case;
end if;
end process;
				
		
			


end architecture;