
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.wishbone_pkg.all;

entity xwb_supervisor_reg is
  port (
    rst_i     : in std_logic;
    clk_sys_i : in std_logic;


    -- Wishbone regs
    slave_i       : in  t_wishbone_slave_in;
    slave_o       : out t_wishbone_slave_out;
    start_recovery_o : out std_logic;
    cpu_status_i  : in  std_logic_vector(2 downto 0);
    boot_done_o : out std_logic

    );
end xwb_supervisor_reg;

architecture rtl of xwb_supervisor_reg is

  signal ack_sreg        : std_logic_vector(9 downto 0);
  signal rddata_reg      : std_logic_vector(31 downto 0);
  signal wrdata_reg      : std_logic_vector(31 downto 0);
  signal bwsel_reg       : std_logic_vector(3 downto 0);
  signal rwaddr_reg      : std_logic_vector(0 downto 0);
  signal ack_in_progress : std_logic;
  signal wr_int          : std_logic;
  signal rd_int          : std_logic;
  signal allones         : std_logic_vector(31 downto 0);
  signal allzeros        : std_logic_vector(31 downto 0);

  signal hot_boot : std_logic;
    attribute syn_radhardlevel        : string;
  attribute syn_radhardlevel of rtl : architecture is "tmr";

begin



-- Main register bank access process.
  process (clk_sys_i, rst_i)
  begin
    if (rst_i = '1') then
      ack_sreg        <= "0000000000";
      ack_in_progress <= '0';
      rddata_reg      <= "00000000000000000000000000000000";
      start_recovery_o   <= '0';
      hot_boot <= '0';
    elsif rising_edge(clk_sys_i) then
-- advance the ACK generator shift register
      ack_sreg(8 downto 0) <= ack_sreg(9 downto 1);
      ack_sreg(9)          <= '0';
      start_recovery_o <= '0';
      if (ack_in_progress = '1') then
        if (ack_sreg(0) = '1') then
          ack_in_progress <= '0';
        else
        end if;
      else      
        if ((slave_i.cyc = '1') and (slave_i.stb = '1')) then
          if (slave_i.we = '1') then
            start_recovery_o <= slave_i.dat(1);
            if (slave_i.dat(0) = '1') then
              hot_boot <= '1';
             end if;
           
          end if;
          rddata_reg(0)  <= hot_boot;
          rddata_reg(15 downto 1) <= (others => '0');
          rddata_reg(18 downto 16) <= cpu_status_i;
          rddata_reg(31 downto 19) <= (others => '0');
          ack_sreg(0)             <= '1';
          ack_in_progress         <= '1';
        end if;
      end if;
    end if;
  end process;

boot_done_o <= hot_boot;

-- Drive the data output bus
  slave_o.dat   <= rddata_reg;
  slave_o.stall <= (not ack_sreg(0)) and (slave_i.stb and slave_i.cyc);
  slave_o.err   <= '0';
  slave_o.rty   <= '0';
-- ACK signal generation. Just pass the LSB of ACK counter.
  slave_o.ack   <= ack_sreg(0);
  slave_o.int <= '0';
end rtl;