library ieee;
use ieee.std_logic_1164.all;

library work;
use work.wishbone_pkg.all;

entity xwb_uart is

  port(
    clk_sys_i : in std_logic;
    rst_i   : in std_logic;

    -- Wishbone
    slave_i : in  t_wishbone_slave_in;
    slave_o : out t_wishbone_slave_out;
    desc_o  : out t_wishbone_device_descriptor;

-- Port for BIT field: 'TX busy' in reg: 'Status Register'
    uart_sr_tx_busy_i                        : in     std_logic;
-- Port for MONOSTABLE field: 'TX trigger' in reg: 'Status Register'
    uart_sr_tx_send_o                        : out    std_logic;
    uart_tdr_tx_data_o                       : out    std_logic_vector(7 downto 0)

    );

end xwb_uart;

architecture rtl of xwb_uart is


component wb_uart
  port (
    rst_n_i                                  : in     std_logic;
    clk_sys_i                                : in     std_logic;
    wb_adr_i                                 : in     std_logic_vector(0 downto 0);
    wb_dat_i                                 : in     std_logic_vector(31 downto 0);
    wb_dat_o                                 : out    std_logic_vector(31 downto 0);
    wb_cyc_i                                 : in     std_logic;
    wb_sel_i                                 : in     std_logic_vector(3 downto 0);
    wb_stb_i                                 : in     std_logic;
    wb_we_i                                  : in     std_logic;
    wb_ack_o                                 : out    std_logic;
    wb_err_o                                 : out    std_logic;
    wb_rty_o                                 : out    std_logic;
    wb_stall_o                               : out    std_logic;
-- Port for BIT field: 'TX busy' in reg: 'Status Register'
    uart_sr_tx_busy_i                        : in     std_logic;
-- Port for MONOSTABLE field: 'TX trigger' in reg: 'Status Register'
    uart_sr_tx_send_o                        : out    std_logic;
-- Port for std_logic_vector field: 'Transmit data' in reg: 'Transmit data register'
    uart_tdr_tx_data_o                       : out    std_logic_vector(7 downto 0)
  );
end component;
attribute syn_radhardlevel : string;
attribute syn_radhardlevel of rtl : architecture is "tmr";

begin



  U_Wrapped_UART: wb_uart

    port map (
      clk_sys_i  => clk_sys_i,
      rst_n_i    => not rst_i,
      wb_adr_i   => slave_i.adr(2 downto 2),
      wb_dat_i   => slave_i.dat,
      wb_dat_o   => slave_o.dat,
      wb_cyc_i   => slave_i.cyc,
      wb_sel_i   => slave_i.sel,
      wb_stb_i   => slave_i.stb,
      wb_we_i    => slave_i.we,
      wb_ack_o   => slave_o.ack,
      wb_stall_o => slave_o.stall,
      uart_sr_tx_busy_i => uart_sr_tx_busy_i,
      uart_sr_tx_send_o => uart_sr_tx_send_o,
      uart_tdr_tx_data_o => uart_tdr_tx_data_o);

  slave_o.err <= '0';
  slave_o.rty <= '0';
  slave_o.int <='0';

  desc_o <= (others => '0');


end architecture;